/*******************************************************************
 *
 *    FILE:			sbdown.c
 *
 *    DESCRIPTION:	
 *			Contains all the routines for downloading the DSP code
 *			to a SoftBoard.
 *
 *			These routines are platform independent, although they
 *			do call three platform-dependent routines: SBTrace(), 
 *			SBReleaseControl(), and SB_PutString(). SBTrace() and 
 *			SBReleaseControl() can be found in SBHW.C, while
 *			SB_PutString() has its prototype in SB_OUT.H. 
 * 
 *			SB_PutString() is only an example way of updating the 
 *			user display. However, in most cases, SBDownShowProgress() 
 * 			will be changed to call an application-specific routine 
 *			that updates the display. SBDownShowProgress() can also be 
 *			moved to another file without changing anything, as the 
 *			constants it uses and its prototype are already in SBDOWN.H.
 *
 *    HISTORY:    	
 *		June 3, 1994:	First released - Guy Townsend
 *		
 *		June 30, 1994:	Updated SBDownShowProgress() to use SB_PutString(),
 *						which is declared in SB_OUT.H. - GAT
 *
 *		July 28, 1994:	Fixed bug in first download, which returned true,
 *						even if the board was turned off.
 *
 *		August 18, 1994: Fixed bug that returned the wrong line number 
 *						 of the file being downloaded if a failure
 *						 occured.    
 *
 *		August 30, 1994: Added a SBTrace statement to SetStatusText()
 *						 to assist in debugging. Also added debugging
 *						 call to WriteByte().
 *						
 *		Sept. 6, 1994	Changed method of resetting the SoftBoard
 *						to send a true break signal, rather than faking
 *						it with a NULL at a very low baud rate. Uses the
 *						new SBCommSendBreak() function.
 *						
 *		Sept. 22, 1994	Fixed problem with sscanf pattern working 
 *						differently for different compilers with
 *						the definition of a HEXPATTERN macro. Also
 *						changed some types of var addresses passed to
 *						sscanf(), which was causing strange errors
 *						with some compilers.
 *
 *						Cleaned up error handling in case of no
 *						handshake char returned for phases 2 and 3.
 * 
 *		Oct. 10, 1994:	Changed sequence of handshaking and baud rate
 *						changes at the end of third download to make
 *						the download procedure more robust. This 
 *						required changes to the DSP code itself, so 
 *						the proper version of SOFTBD.DSP must be
 *						used.
 *
 *		Dec. 8, 1994:	Changed use of sscanf to local HexToInt function
 *						to eliminate problems with compiler bugs as well
 *						as sscanf() not being supported in Windows DLL
 *						code under Visual C++ 1.5. Required removing the
 *						addition of "0x" to the beginning of each hex number. 						
 *
 *						Moved SBDownShowProgress() to its own file, 
 *						called progress.c
 *
 *		Jan. 11, 1994:	Cleaned up error strings by using strings
 *						defined in sberrstr.h.	
 *
 *						Added return() calls to SBDownload when files are 
 *						not found, so that the function fails and returns 
 *						on the first file that is not found. 
 *
 *						Cleaned up handling of return at end of download 1.
 *						
 *		Jan. 12, 1994:	Changed to use new hex files that eliminate
 *						high resolution timing. Rather than using a
 *						G or B sent back from the board, a 0xFF is used, 
 *						which leaves the DSP always ready to receive
 *						the next command from us. 
 *
 *		Apr. 26, 1995:	Changed all sprintf calls to SBSPrintfXXX calls
 *						located in sbhw.c so that a Windows DLL using 
 *						this file would be safe. Also changed some
 *						calls (which use constants from sberrstr.h) 
 *						from strcpy to SBStrcpy.
 *
 *	
 *******************************************************************/

/** include files **/
#include "sbhw.h" 
#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <unistd.h>
#include <linux/in.h>
#include <string.h>    
#ifdef SB_UNIX
	#include <sys/stat.h>
#else
	#include <sys\stat.h>
#endif
#include <stdlib.h>
#include <assert.h> 

#include "sbdown.h"
#include "sbcomm.h"
#include "progress.h"
#include "sberrstr.h"

/* DOS version uses underscores for some system calls */
#ifdef SB_UNIX
	#define _stat stat
#endif

/** global data **/
long Phase1Size;	/* size of first download file */
long Phase2Size;	/* size of second download file */
long Phase3Size;	/* size of third download file */
long TotalSize;		/* total size of all files to be downloaded */
long NumRead;		/* total download file bytes read */
int PercentDone;	/* percent of the total file volume read */
char StatusString[SB_MAX_STR_LEN];

/** local functions **/
void SBSprintfStr(char *buf, char *err1, char *err2);
void SBSprintfInt(char *buf, char *err1, int err2);
void SBSprintfStrInt(char *buf, char *err1, char *err2, int err3);
void SBDownShowProgress(int a, int b);
BOOL Download1(char *Filename, char *ErrorString);
BOOL Download2(char *FileName, BOOL Phase3, char *ErrorString);
void SetStatusText( char *str);
void SetErrorText( char *str);
BOOL IsCommLinkError(char *ErrorString);
BYTE MakeByte(char *TextPoint);
void SlowWriteByte(BYTE b);
void WriteByte(BYTE b);
void WriteWord2(unsigned short word);
int ReadByte(void);
int HexToInt(char *Text); 

void SBDownShowProgress(int a, int b)
{
  fprintf(stderr, "Phase %d: %d\%\r", a, b);
}

void SBSprintfStr(char *buf, char *err1, char *err2)
{
  sprintf(buf, "%s: %s", err1, err2);
}

void SBSprintfInt(char *buf, char *err1, int err2)
{
  sprintf(buf, "%s: %d", err1, err2);
}

void SBSprintfStrInt(char *buf, char *err1, char *err2, int err3)
{
  sprintf(buf, "%s: %s %d", err1, err2, err3);
}



/*---------------------------------------------------------------------------
 ** SBDownload()
 
 *  PARAMETERS:		
		Phase1FileName: The first file to be downloaded to the SBoftBoard. 
		Phase2FileName: The second file to be downloaded. 
		Phase3FileName: The third and last file to be downloaded.
		ErrorString:	Buffer that will contain a description of any error
						that occurs.

 *  DESCRIPTION:	
		This function initiates a download of the three needed files to the
		SoftBoard's DSP. The serial port is expected to already have been
		opened using the SBCommOpen function. 
 
 *  RETURNS:
		TRUE on success. FALSE on failure, with the error in ErrorString.
 		
--------------------------------------------------------------------------*/
BOOL SBDownload( 	char *Phase1FileName, char *Phase2FileName,
					char *Phase3FileName, char *ErrorString )
{    
	
	int c, result;
	struct _stat filestat;     
	char buf[SB_MAX_STR_LEN]; /* temp buf for error string construction */

	NumRead = 0;
	SBAbortFlag = FALSE;

	/* make sure the files to be downloaded can be found, */
	/* and get their sizes */
	if ( _stat(Phase1FileName, &filestat) == 0 ) 
	{
		Phase1Size = (long) filestat.st_size;
	}
	else
	{
		SBSprintfStr( buf, SBERR_FILE_NOT_FOUND, Phase1FileName);
		strncpy(ErrorString , buf, SB_MAX_STR_LEN);
		return(FALSE);
	}

	if ( _stat(Phase2FileName, &filestat) == 0 ) 
	{
		Phase2Size = (long) filestat.st_size;
	}			
	else
	{
		SBSprintfStr( buf, SBERR_FILE_NOT_FOUND, Phase2FileName);
		strncpy(ErrorString , buf, SB_MAX_STR_LEN);
		return(FALSE);
	}
	
	if ( _stat(Phase3FileName, &filestat) == 0 ) 
	{
		Phase3Size = (long) filestat.st_size;
	}
	else
	{
		SBSprintfStr( buf, SBERR_FILE_NOT_FOUND, Phase3FileName);
		strncpy(ErrorString , buf, SB_MAX_STR_LEN);
		return(FALSE);
	}

	TotalSize = Phase1Size + Phase2Size + Phase3Size;
	PercentDone = 0;
	SB_PutString("Flushing Comm buffers");
	//	SetStatusText("Flushing Comm buffers");
	SBCommClearError();
	SBCommFlush();
	if ( IsCommLinkError(ErrorString) ) 
	  {
	    return(FALSE);
	  }
	/* The SoftBoard is reset by sending a break signal on the 
		serial port data line. The break signal is about 250 
		milliseconds, which is about 5 times what is required. */
	SB_PutString("Resetting SoftBoard");
	/*	SetStatusText("Resetting SoftBoard"); */
	SBCommSendBreak();
	if ( IsCommLinkError(ErrorString) ) 
		return(FALSE);

	/* the first phase of the download is at 2400 baud */
	SB_PutString("Setting initial baud rate");
	//	SetStatusText("Setting initial baud rate");
	SBWait(150);
	SBCommSetBaudRate(2400);
	if ( IsCommLinkError(ErrorString) ) 
		return(FALSE);

	SBCommFlush();
	/* send the first file to the board */
	SB_PutString("Downloading File 1");
	//	SetStatusText("Downloading File 1");
	if ( !Download1( Phase1FileName, ErrorString ) ) 
		return FALSE; 
	if ( IsCommLinkError(ErrorString) ) 
		return(FALSE);
	NumRead = Phase1Size;
	SBDownShowProgress(SHOW_DOWNLOAD1_SUCCESS,0); /* tell the user */
    SBReleaseControl(); /* let the OS catch up and update the screen */

    /* the second and third phases occur at 19200, to speed up
       the process */
	SBCommSetBaudRate(19200);
	SBCommFlush();

	/* send the second file to the board */
	SetStatusText("Downloading File 2");
	if ( !Download2(Phase2FileName, FALSE, ErrorString) ) 
		return FALSE;
	if ( IsCommLinkError(ErrorString) ) 
		return(FALSE);
	result = SBCommByteReadyWait(3000); /* wait for the byte we're expecting */
	if (result < 1)
	{
		if( result == COMM_TIMEOUT)
		{
	    	strcpy(	ErrorString, SBERR_PHASE2_NO_HANDSHAKE );
			return FALSE;
		}
		else
		{
			if ( IsCommLinkError(ErrorString) ) 
				return(FALSE);
	    	SBSprintfInt(ErrorString, SBERR_PHASE2_UKNOWN_COMM_ERR, result);
			return FALSE;
		}
	}
	/* else we have a byte ready */	
	 c = SBCommReadByte();
/* xxx_new */
	if(c != 0xff) 
	{
		if ( IsCommLinkError(ErrorString) ) 
			return(FALSE);
		else
		{
			SBSprintfInt(ErrorString, SBERR_PHASE2_BAD_HANDSHAKE, c);
			return FALSE;
		}
    }
	NumRead += Phase2Size;
	SBDownShowProgress(SHOW_DOWNLOAD2_SUCCESS,0); /* tell the user */
    SBReleaseControl(); /* let the OS catch up and update the screen */

	/* send the third and last file to the board */
	SetStatusText("Downloading File 3");
	if ( !Download2(Phase3FileName, TRUE, ErrorString) ) 
		return FALSE;    
	if ( IsCommLinkError(ErrorString) ) 
		return(FALSE);
 	result = SBCommByteReadyWait(3000); /* wait for the byte we're expecting */
	if (result < 1)
	{
		if( result == COMM_TIMEOUT)
		{
	    	strcpy(	ErrorString, SBERR_PHASE3_NO_HANDSHAKE);
			return FALSE;
		}
		else
		{
			if ( IsCommLinkError(ErrorString) ) 
				return(FALSE); /* ErrorString gets returned */
			else
			{
		    	SBSprintfInt(ErrorString, SBERR_PHASE3_UKNOWN_COMM_ERR,	result);
				return FALSE;
			}
		}
	}
	/* else we have a byte ready */	
	 c = SBCommReadByte();
	if(c != 'G') /* this is from SOFTBD.DSP */
	{
		if ( IsCommLinkError(ErrorString) ) 
			return(FALSE);
		else
		{
			SBSprintfInt(ErrorString, SBERR_PHASE3_BAD_HANDSHAKE, c);
			return FALSE;
		}
    }

   	/* If we get here then all three downloads have been successful.
   		Now we need to change baudrates to our normal operating rate 
   		of 9600. */
	SBDownShowProgress(SHOW_DOWNLOAD3_SUCCESS,0); /* tell the user */
	result = SBCommSetBaudRate(9600);
	if (result != COMM_NO_ERROR)
	{
		if( result == COMM_TIMEOUT)
		{
	    	strcpy(ErrorString, SBERR_BAUDSET_TIMEOUT);
			return FALSE;
		}
		else
		{
			if ( IsCommLinkError(ErrorString) ) 
				return(FALSE);
			else
			{
		    	SBSprintfInt(ErrorString, SBERR_BAUDSET, result);
				return FALSE;
			}
		}
	}
	SBCommFlush();
	
	/* At this point, the SoftBoard will patiently wait for us before it
		does anything else. We have to send a byte to tell the DSP code to 
		actually start running. The SoftBoard should respond by sending 
		yet another 'G' back to us. Just to be really safe, we'll wait 
		here a bit to make sure the 'G' (including the stop bits) has
		been fully sent by the DSP. */
	SBWait( 100 );
	SBCommWriteByte(0x00);
	if ( IsCommLinkError(ErrorString) ) 
		return(FALSE);
	 	 
	result = SBCommByteReadyWait(3000); /* wait for the byte we're expecting */
	if (result < 1)
	{
		if( result == COMM_TIMEOUT)
		{
	    	strcpy(ErrorString,	SBERR_SBSTART_NO_HANDSHAKE );
			return FALSE;
		}
		else
		{
			if ( IsCommLinkError(ErrorString) ) 
				return(FALSE);
			else
			{
		    	strcpy(	ErrorString,SBERR_SBSTART_UKNOWN_COMM_ERR );
				return FALSE;
			}
		}
	}
	/* else we have a byte ready */	
	 c = SBCommReadByte();
	if(c != 'G') /* this is from SOFTBD.DSP */ 
	{
		if ( IsCommLinkError(ErrorString) ) 
			return(FALSE);
		else
		{
			SBSprintfInt(ErrorString, SBERR_SBSTART_BAD_HANDSHAKE, c);
			return FALSE;
		}
    }

	return TRUE;
} /* end Download */


/*---------------------------------------------------------------------*/
/*                     FirstDownload                                   */
/*---------------------------------------------------------------------*/
BOOL Download1(char *Filename, char *ErrorString)
{
	int c, i, result;
	unsigned short status;
	unsigned short len2;
    unsigned short 	checksum	= 0;
 	int type = 0;
	unsigned short Low, High;
	char *TextPoint;
    FILE *infile;
	int Count; /* count of the bytes per line in the hex file */
	int Address;
	unsigned short Word;
	char buff[129];
	char tbuff[129];
	short DataN = 0;
	unsigned short *Mem;

	//	SBTrace("starting first download\n");
	SB_PutString("Starting first download");
	/* allocate a buffer to hold the data (this file is really small) */
	/* this will be 8k, with 16 bit ints, which is more than enough */
	Mem = (unsigned short *) malloc( sizeof(unsigned) * 0x1000); 

	/* open the file in read-only text mode */
	infile = fopen( Filename, "rt");
	if(infile == NULL) 
	{   
	  SB_PutString("Failed to open file #1");
		SBSprintfStr(ErrorString, SBERR_FILE_CANNOT_OPEN, Filename);
		return FALSE;    
	}

    /* read the file into a memory buffer */
	while(type != 1)
	{
		fgets(buff, 128, infile);
		if(feof(infile))
		{   
		  SB_PutString("Premature end of file");
            SBSprintfStr(ErrorString, SBERR_FILE_INVALID, Filename );
            goto exit_on_error;
		}
		if(buff[0] != ':') continue; /* skip over embedded text */
		strncpy(tbuff, buff + 7, 2);
		tbuff[2] = 0;
		type = HexToInt( tbuff);
		if(type == 1) break;

		strncpy(tbuff, buff + 1, 2);
		Count = HexToInt( tbuff);
		Count >>= 1;

		strncpy(tbuff, buff + 3, 4);
		tbuff[4] = 0;
		Address = HexToInt( tbuff);
		if(Address < 0xFA00) continue;

		TextPoint = buff + 9;
                 
		for(i = 0; i != Count; i++)
		{
			High = (unsigned short) MakeByte(TextPoint);
			TextPoint += 2;
			Low = (unsigned short) MakeByte(TextPoint);
			TextPoint += 2;
			Word = (High <<  8) | Low;
			Mem[DataN++] = Word; 
		}
		
		/* 	let the rest of the system run, which may
		mean that the user pushes the Abort button */
		if(	SBReleaseControl() )
		{
		  SBSprintfStr(ErrorString, SBERR_USER_ABORT, Filename);
			goto exit_on_error;
		}

	}
	fclose(infile);
	infile = 0;  

    if( DataN == 0 ) 
    {
      free(Mem);
    	SBSprintfStr(ErrorString, SBERR_FILE_INVALID, Filename);
    	goto exit_on_error;
    } 
    
    if( DataN > 256 ) 
    {
    	free(Mem);
    	SBSprintfStr(ErrorString, SBERR_FILE_INVALID, Filename);
    	goto exit_on_error;
    } 
    

	/* send code to the board */
	if ( IsCommLinkError(ErrorString) ) 
    	goto exit_on_error;
	len2 = (unsigned short) DataN - 1;
	SlowWriteByte(0xff);  /* baud rate detect byte */
	status = 0x08 | (len2 >> 8);
	SlowWriteByte( (BYTE) status);
	SlowWriteByte(0x40);     /* Interrupt word. */
	SlowWriteByte( (BYTE)(len2 & 0xff) );
	if ( IsCommLinkError(ErrorString) ) 
	  {
	    goto exit_on_error;
	  }
	for(i = 0; i != DataN; i++) 
	{
		checksum += Mem[i];
		WriteWord2( Mem[i] );
	}
	SlowWriteByte( (BYTE)(checksum & 0x00ff) );
	SlowWriteByte( (BYTE)(checksum >> 8) );
	SlowWriteByte(0);

	if ( IsCommLinkError(ErrorString) ) 
	  {
	    goto exit_on_error;
	  }

   	
   	free(Mem);

	/* wait for handshaking byte from board */
	result = SBCommByteReadyWait(300000); 
	if (result < 1)
	{
		if( result == COMM_TIMEOUT)
		{
		  strcpy(ErrorString,	SBERR_PHASE1_NO_HANDSHAKE );
		  return FALSE;
		}
		else
		{
			if ( IsCommLinkError(ErrorString) ) 
				return(FALSE);
			else
			{
		    	strcpy(	ErrorString, SBERR_PHASE1_UKNOWN_COMM_ERR );
				return FALSE;
			}
		}
		}
	/* else we have a byte ready */	
	c = SBCommReadByte();
/* xxx_new */
	if(c != 0xff) 
	{
		if ( IsCommLinkError(ErrorString) ) 
			return(FALSE);
		else
		{
			SBSprintfInt(ErrorString, SBERR_PHASE1_BAD_HANDSHAKE, c);
			return FALSE;
		}
    }
	else /* we did recieve a 'G' */
	{
		SBTrace("first download succesful\n");
		return TRUE;
	}

		
exit_on_error:
	if(infile != 0)
		fclose(infile);
	return FALSE;

} /* Download1() */


/*---------------------------------------------------------------------*/
/*                  Second and Third Downloads                         */
/*---------------------------------------------------------------------*/
BOOL Download2(char *FileName, BOOL Phase3, char *ErrorString)
{
	int type = 0;
	int c;
	unsigned short Low, High;
	char *TextPoint;
	int i, j;
	unsigned short CheckSum;
    FILE *infile;
	int Count;    /* count of the bytes per line in the hex file */   
	int LineCount;
	int UpdateFlag;
	unsigned short StartAddress;
	unsigned short Address;
	unsigned short Word;
	char buff[129];
	char tbuff[129];
	BYTE writebuff[60];
	int HandShakeByte;

	if ( Phase3 )
	{
		StartAddress = 0xfa10;
		SBTrace("starting download 3\n");
	}
	else /* phase 2 */
	{
		StartAddress = 0x8000;
		SBTrace("starting download 2\n");
    }
    
	infile = fopen(FileName, "rt");
	if(infile == NULL) 
	{
		SBSprintfStr(ErrorString, SBERR_FILE_NOT_FOUND, FileName);
        return FALSE;
    }
    
	CheckSum = 0;
	LineCount = 0;

	while(type != 1)
	{
		fgets(buff, 128, infile);
		if(feof(infile)) 
		{
			SBSprintfStr(ErrorString, SBERR_FILE_INVALID, FileName);
			goto exit_on_error;
		}

		if(buff[0] != ':') continue; /* skip over embedded text */

		LineCount++;
		/* allow updates to the user every 10 lines (about 1% of total) */
		if(LineCount % 10)
		{
			UpdateFlag = TRUE;
		}
		else
			UpdateFlag = FALSE;
			
		if( Phase3 ) 
		{
			NumRead += strlen( buff );
			if( UpdateFlag )
				SBDownShowProgress( SHOW_PERCENT,
								(int)((NumRead*100)/TotalSize) );
 		}
		
		/* 	let the rest of the system run, which may
			mean that the user pushes the Abort button */
		if(	SBReleaseControl() )
		{
	    	SBSprintfStr(ErrorString, SBERR_USER_ABORT, FileName);
			goto exit_on_error;
		}

Retry:
		strncpy(tbuff, buff + 7, 2);
		tbuff[2] = 0;
		type = HexToInt( tbuff);
		if(type == 1) break;

		strncpy(tbuff, buff + 1, 2);
		Count = HexToInt( tbuff);
		Count >>= 1;
		strncpy(tbuff, buff + 3, 4);
		tbuff[4] = 0;
		Address = (unsigned short) HexToInt( tbuff);
		/* 	If the address is below 0x8000 we will have to do a 
			bank switch to run it, so we will load it high for now.
   			If the Address is at or above 0xfa00 then we will load it 
   			into data area 0x200 and then switch it to program memory 
   			(this will remap it to 0xfa00).
		*/		
		if(Address < 0x8000)
			Address += 0x8000;
		else
		{
			if(Address > 0xf0ff)
				Address -= (0xfa00 - 0x200);
		}  
		
		if ( IsCommLinkError(ErrorString) ) 
    		goto exit_on_error;
    		
		WriteByte(0x1b);
		WriteByte('B');
		c = SBCommByteReadyWait(1000);  
		if( c < 0)
		{
			if (c == COMM_TIMEOUT)	
			{
				SBSprintfStrInt(ErrorString, SBERR_NO_START_HANDSHAKE, FileName, LineCount);
				goto exit_on_error;
			}
			/* else it is some other error, so we want to find out
			   which one and get the ErrorString set appropriately */
			if( IsCommLinkError(ErrorString) )
				goto exit_on_error;
		}
		c = SBCommReadByte(); 
/* xxx_new */
		if(c != 0xff) 
		{
			if ( IsCommLinkError(ErrorString) ) 
				goto exit_on_error;
			SBSprintfStrInt(ErrorString, SBERR_BAD_HANDSHAKE, FileName, c );
			goto exit_on_error;
		}
/* xxx_new */
/*		SBWait(1); */
		WriteWord2(Address);
		WriteByte((BYTE) (0xff & Count));
		CheckSum = Address + Count;
		TextPoint = buff + 9;

		for(i = 0, j = 0; i != Count; i++) 
		{
			High = MakeByte(TextPoint);
			TextPoint += 2;
			Low = MakeByte(TextPoint);
			TextPoint += 2;
			Word = (High <<  8) | Low;
			CheckSum += Word;
			
			/* store in a term array and then send, so as to minimize
			   OS accesses to the comm port. */
			writebuff[j++] = (BYTE)(Word & 0x00ff);
			writebuff[j++] = (BYTE)(Word >> 8);
		}
		SBCommWriteData(writebuff, j);
		WriteWord2(CheckSum);
 
		if ( IsCommLinkError(ErrorString) ) 
    		goto exit_on_error;
		c = SBCommByteReadyWait(1000);
		if (!c) 
		{
			SBSprintfStrInt(ErrorString, SBERR_NO_END_HANDSHAKE, FileName, LineCount);
			goto exit_on_error;
		}
		HandShakeByte = SBCommReadByte();
/* xxx_new */
		/* SBWait(1); */
		if(HandShakeByte != 0xff) 
		{
			SBTrace("retry handshaking byte ..\n");
			goto Retry;
		}

	} /* end of while loop */

	fclose(infile);

	/* special case for the last update */
	if( Phase3 ) 
		SBDownShowProgress(SHOW_PERCENT, (int)((NumRead*100)/TotalSize) ); 
	
	SBTrace(" download sent\n");
	if( Phase3 ) 
		SBDownShowProgress(SHOW_PERCENT, 100);

	/* send command to the board to jump to the start of the 
	   code just sent */
	WriteByte(0x1b);
	WriteByte('G');
	WriteWord2(StartAddress);
	WriteByte(0);
	if ( IsCommLinkError(ErrorString) ) 
   		return(FALSE);

	return TRUE;
 
exit_on_error:
	fclose(infile);
	SBTrace(" download failed\n");
	return(FALSE);
}


/**********************************************************
				utility routines
**********************************************************/


void SetStatusText( char *str)
{
	SBTrace(str);
	strncpy( StatusString, str, SB_MAX_STR_LEN );
}


BOOL IsCommLinkError(char *ErrorString)
{
	char temp[SBCOMM_ERR_STR_MAX];
	
	if ( SBCommIsError() ) 
	  {
		SBCommGetErrorString(temp);
		SBCommClearError();

		strncpy( ErrorString, StatusString, SB_MAX_STR_LEN);
		strcat( ErrorString, " : " );
		strcat( ErrorString, temp );
		SB_PutString(ErrorString);
		return(TRUE);
	  }
	return(FALSE);
}  


BYTE MakeByte( char *TextPoint)
{
	BYTE byte;

	if(*TextPoint > '9')
		byte = (BYTE)((*TextPoint - 'A' + 10) << 4);
	else
		byte = (BYTE)((*TextPoint - '0') << 4);

	TextPoint++;
	if(*TextPoint > '9')
		byte |= *TextPoint - 'A' + 10;
	else
		byte |= *TextPoint - '0';

	return byte;
}


void SlowWriteByte(BYTE b)
{
	SBWait(1000);
	WriteByte(b);
}


void WriteByte(BYTE b)
{
	int result = SBCommWriteByte(b);
	counter++;
	if (!(counter % 200))
	  {
	    strcpy(outmesg, "DOWNLOAD_DSP");
	    send(outsocket, outmesg, strlen(outmesg), 0);
	    while (1)
	      {
		int len;
		if (len=recv(inclient, inmesg, sizeof(inmesg), 0))
		  {
		    inmesg[len]='\0';
		    if (!strcmp(inmesg, "DOWNLOAD_DSP_READY"))
		      {
			break;
		      }
		  }
	      }
	  }
	if(result != COMM_NO_ERROR)
		SBTrace("Error writing byte");
	
}


void WriteWord2(unsigned short word)
{
	WriteByte( (BYTE)(word & 0x00ff) );
	WriteByte( (BYTE)(word >> 8) );
}




int ReadByte(void)
{
	int c;
	     
	c = SBCommByteReadyWait(1000); 
	if (c < 0 )
		return -1;
	return ( SBCommReadByte() );
}


int HexToInt(char *Text)
{
	int Num = 0;
	int i = 0;
	while(Text[i])
	{
		Num <<= 4;
		if(Text[i] > '9')
			Num |= Text[i] - 'A' + 10;
		else
			Num |= Text[i] - '0'; 
		i++;
	}
	return Num;
}
		





