/*******************************************************************
 *
 *    FILE:			loop.c
 *
 *    DESCRIPTION:	
 *			A sample routine that exercises the SBComm routines
 *			with a loopback plug on the serial port. The loopback
 *			plug only needs the transmit and recieve data pins
 *			shorted.
 *			
 *			This routine should	help confirm that basic serial port 
 *			functionality is present. 
 *
 *			The SB_ShowError(), SB_ClearOutputWindow and SB_PutString()
 *			functions are used for all output to the screen and can be 
 *			changed for other platforms. These functions have prototypes
 *			in sbout.h.
 *
 *			The functions in this file could easily be included in 
 *			the main program, but are in this separate file to separate 
 *			the platform-independent code in this file from the 
 *			application-specific code.
 *
 *    HISTORY:    	
 *		June 3, 1994:	First released - Guy Townsend
 *
 *		June 13, 1994:	Changed the output functions to call functions
 *						in an improved Windows shell program. Added
 *						blank lines between sections in the output. 
 *
 *		June 30, 1994:	Changed to support sb_out.h functions.
 *
 *		Sept. 8, 1994:	Changed LoopBackTest parameter to a string,
 *						to be consistent with the SBCommOpen() change.
 *
 *		Sept. 22, 1994:	Added conditional statements to change
 *						the block size for UNIX development where
 *						large blocks of data on a loopback plug 
 *						seemed to cause a problem.
 *
 *******************************************************************/

/** include files **/
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "loop.h"
#include "sbcomm.h"
#include "sbout.h"		/* generic output routines  */

#ifdef SB_UNIX
	#define TEST_SIZE			30
	#define SMALL_TEST_SIZE 	30
#else
	#define TEST_SIZE			1024
	#define SMALL_TEST_SIZE 	255
#endif    
                            
/* global data */
unsigned char OutData[ TEST_SIZE ];
unsigned char InData[ TEST_SIZE ];
char ErrBuff[SBCOMM_ERR_STR_MAX]; /* error string buffer for all functions */

/** internal functions **/
BOOL TestByte();
BOOL TestBlock();
BOOL TestBlock2();
void Flush();

BOOL bWorking;

/** external program calls here to do the loopback test **/
int LoopBackTest( char * PortString )
{
	int result;
	
	/* set a flag to prevent re-entrancy */
	if(bWorking)
		return 0;
	else
		bWorking = TRUE;

	SB_ClearOutputWindow(); /* clears the results window */

	SB_PutString("Beginning Test");

	/* open the comm port */
	result = SBCommOpen( PortString );
	if (result != COMM_NO_ERROR)
	{
		SBCommGetErrorString(ErrBuff);
		SB_ShowError(ErrBuff);
		SBCommClearError();
    	goto CLOSE_PORT;
	}
	SB_PutString("Opened Comm Port");
	/*  flush the buffers */
    Flush();
 
 	/*  set some different baud rates, 
		and test at each speed  */
	result = SBCommSetBaudRate(2400);
	if (result != COMM_NO_ERROR)
	{
		SBCommGetErrorString(ErrBuff);
		SB_ShowError(ErrBuff);
		SBCommClearError();
    	goto CLOSE_PORT;
	}
	SB_PutString("\n");
	SB_PutString("Set Baud Rate to 2400");
    Flush();
    if( TestByte() == FALSE)
    	goto CLOSE_PORT;
    if( TestBlock() == FALSE)
    {
		goto CLOSE_PORT;
    }
 	/* don't do TestBlock2 here, becuase it expects a faster baud rate
 		when calculating how long to wait for the block */
 
	result = SBCommSetBaudRate(9600);
	if (result != COMM_NO_ERROR)
	{
		SBCommGetErrorString(ErrBuff);
		SB_ShowError(ErrBuff);
		SBCommClearError();
		bWorking = FALSE;
		return(result);
	}
	SB_PutString("\n");
	SB_PutString("Set Baud Rate to 9600");
    Flush();
    if( TestByte() == FALSE)
    	goto CLOSE_PORT;
    if( TestBlock() == FALSE)
    {
		goto CLOSE_PORT;
    }
    if( TestBlock2() == FALSE)
    {
    	goto CLOSE_PORT;
    }

	result = SBCommSetBaudRate(19200);
	if (result != COMM_NO_ERROR)
	{
		SBCommGetErrorString(ErrBuff);
		SB_ShowError(ErrBuff);
		SBCommClearError();
    	goto CLOSE_PORT;
	}
	SB_PutString("\n");
	SB_PutString("Set Baud Rate to 19200");
    Flush();
    if( TestByte() == FALSE)
	{
	   	goto CLOSE_PORT;
    }
    if( TestBlock() == FALSE)
    {
    	goto CLOSE_PORT;
    }
    
    TestBlock2();
    
CLOSE_PORT:
	SB_PutString("\n");
	bWorking = FALSE;	/* reset the non re-enterency flag */
	/*  close the port */
	if( SBCommClose() != 0)
	{
		SB_ShowError("!! Error closing comm port");
		return(COMM_IE_BADID);
	}	
    SB_PutString("Closed Comm Port.");
	
	SB_PutString(" Ending Test");
    return( COMM_NO_ERROR );
}


/*  Flush the buffers. */
/*  Make a function because we want to use it multiple times above. */
void Flush()
{
	SBCommFlush();
	if ( SBCommIsError() )
	{
		SBCommGetErrorString(ErrBuff);
		SB_ShowError(ErrBuff);
		SBCommClearError();
		SB_PutString("!! Flush of Buffers Failed");
	}                   
	else
		SB_PutString("Flushed Buffers");
}
                            
/**	send 10 bytes of data out the port, and retrieve it 
	using the SBCommReadByte routine. */
BOOL TestByte()
{
	int byte, in, i;
 	char buf[80];
	int error, result;

	error = FALSE;

	for (i = 0; i < 10 ;i++ )
	{
		byte = i + 0x41;
		result = SBCommWriteByte( (unsigned char) byte);
		if(result != COMM_NO_ERROR)
		{
		 	sprintf(buf, "!! Byte %d:  SBCommWriteByte result: %d", i, result);
		 	SB_PutString(buf);
		}
		
		/* wait for the system to send the byte out the port
			(which might not happen right away if the system is busy)
			and then read it again*/
		result = SBCommByteReadyWait(500);
		if( result == COMM_TIMEOUT )
		{
			sprintf(buf, "!! Byte %d:  Timed Out waiting for data",i);
			SB_PutString(buf);
			error = TRUE;
		}
		else if(result >= 0)
		{
			in = SBCommReadByte();
			if(  in != byte)
			{
				sprintf(buf, "Byte %d:  Sent: %X;  Rcvd: %X",i, byte, in);
				SB_PutString(buf);
				error = TRUE;
			}              
		}
		else	/* a real error */	
		{
		 	SBCommGetErrorString(ErrBuff);
		 	SB_ShowError( ErrBuff );
		 	SBCommClearError();
		 	error = TRUE;
		 	break;
		}
		
	}
	if(!error)
	{
		SB_PutString("Byte Test Passed!");
		return(TRUE);
	}
	else
	{
		SB_PutString("!! Byte Test Failed");
		return(FALSE);
	}

}


/**	send TEST_SIZE of data out the port, and retrieve it 
	using the SBCommReadData routine. */
BOOL TestBlock()
{
	int i;         
	int DataReceived;
	unsigned long NoDataCount;
	int error;
	int result;
    char buf[80];
    
	/* fill the buffer with pseudorandom numbers */
	srand( (unsigned)time( NULL ) ); /* seed the generator */
	for( i = 0;   i < TEST_SIZE; i++ )
		OutData[i] = (unsigned char) rand();

	/* send the data out the port */
	SBCommWriteData( OutData, TEST_SIZE );
	
    /* read the data back in */
	DataReceived = 0;
	NoDataCount = 0;
	error = FALSE;
	do
	{
    	/* 	Pass the address of the buffer offset by 
    		the number of bytes receieved.
    		Ask for only the amount left of the data we expect. */
    	result = SBCommReadData( 	InData + DataReceived, 
    								TEST_SIZE - DataReceived );
		/* 	check for errors, but if there was no data available, 
			then just keep going. */
		if(result < 0)
		{
		 	SBCommGetErrorString(ErrBuff);
		 	SB_ShowError( ErrBuff );
		 	SBCommClearError();
		 	error = TRUE;
		 	break;
		}
		else
		{
		/* 	we had no error, so update the amount 
			of data we have collected so far */
			DataReceived += result; 
			if(result == 0)
				NoDataCount++;
			else
				NoDataCount = 0;
		}
		
		/* 	To make sure we don't sit in the loop forever if we never
			receive all the data we expect, we keep a count of how many
			times we get no data when we ask for it. The counter is reset
			whenever we get any data, so this only applies to running out
			of data. This is not the best way, and may need to be adjusted
			for different machines, but it is portable in that it doesn't
			call any platform-specific timer routines. This method is just
			so we don't get stuck forever, and the program will eventually
			stop. */
		if(NoDataCount > 500000)
		{
			SB_ShowError("Breaking out of endless loop");
			error = TRUE;
			break;
		}
				
	}while( DataReceived < TEST_SIZE);
	
	if(error)
	{
		sprintf(buf,"%d bytes expected, only %d bytes received", 
					TEST_SIZE, DataReceived);
		SBTrace(buf);
		SB_PutString("!! Block Test #1 Failed");
		return( FALSE );
	}
	else
	{
		SB_PutString("Block Test #1 Passed");
	    return( TRUE );
    }

}


/**	send SMALL_TEST_SIZE bytes of data out the port, and retrieve it 
	using the SBCommReadDataWait() routine. 
	This is different than the above function in that it
	uses the function that waits for all the expected 
	data. The code here is simplified by using that 
	routine, but the block size is limited to 256 bytes. */
BOOL TestBlock2()
{
	int result, i;
    
	/* fill the buffer with pseudorandom numbers */
	srand( (unsigned)time( NULL ) ); /* seed the generator */
	for( i = 0;   i < SMALL_TEST_SIZE; i++ )
		OutData[i] = (unsigned char) rand();

	/* send the data out the port */
	SBCommWriteData( OutData, SMALL_TEST_SIZE );

	/* Wait for the data to leave the port. */
	/* 4 ms is approximate byte transfer rate at 2400 baud */
	SBWait(SMALL_TEST_SIZE * 4);
	
    /* read the data back in */
   	result = SBCommReadDataWait( InData, SMALL_TEST_SIZE );
	if(result < 0)
	{
	 	SBCommGetErrorString(ErrBuff);
	 	SB_ShowError( ErrBuff );
	 	SBCommClearError();
		SB_PutString("!! Block Test #2 Failed");
		return( FALSE );
	}
	else
	{
		SB_PutString("Block Test #2 Passed");
	    return( TRUE );
    }
}


