/*******************************************************************
 *
 *    FILE:			sbcomm.c
 *
 *    DESCRIPTION:	All serial communication functions for the 
 *					Softboard are in this file. All the other 
 *					example code in the SoftBoard Software
 *					Developer's Kit calls the functions in this
 *					file to access the serial port, and thus the 
 *					SoftBoard.
 *
 *					The design of the comm port interface is 
 *					essentially an object oriented design, except
 *					that instead of using a class, such as in C++, the
 *					functions all use the prefix SBComm. Also, there
 *					are data items that are global to this file, but
 *					not used in other modules. The interface to the
 *					comm port is strictly through the interface 
 *					defined in the sbcomm.h file.
 *
 *				 **	Although the comm interface is defined in sbcomm.h
 * 					as a platform independent-interface, the 
 *					implementation of the functions in this file works with 
 *					for the SCO V/386 version of UNIX, with support added
 *					for NEXSTEP v3.2. For other platforms, you will need 
 *					to replace the internals of the functions with code 
 *					that functions in the same way on your platform.
 *
 *					This code currently uses a secondary receieve buffer
 *					that is separate from the OS comm driver buffer. This was 
 *					origininally done for the Windows implementation, but seems
 *					to work fine for other platforms, and makes the porting 
 *					between a number of different platforms easier for us. 
 *					If the secondary buffer is not desirable, it is not 
 *					difficult to remove it, and the interface to the application
 *					should not need to change.
 *
 *					This code could no doubt be improved to integrate better
 * 					with the multitasking environment, and/or with the
 *					the programs user interface, such as XWindows. However,
 * 					this code works, and seems to be a good starting place.
 *
 *					An attempt was made to use POSIX functions, but both
 *					SCO and NEXTSTEP place serious restrictions on the 
 *					use of the POSIX functions, so ioctls are still used, 
 *					even though the termios struct is used. If you need
 *					POSIX functions, and your system supports them properly,
 *					just search for ioctl calls, and replace with 
 *					tcgetattr, tcsetattr, etc. The SBCommSetBaudRate() may 
 *					require special attention if you need to use the 
 *					cfsetispeed/cfsetospeed functions. Still, it's not many
 *					changes.
 *					
 *
 *    HISTORY:    	
 *		Sept. 8, 1994:	First release of UNIX version - Guy Townsend
 *
 *		Sept. 22, 1994:	Fixed bugs in SBCommReadDataWait and
 *						SBCommReadData.
 *
 *						Added the SBCommPeekMFG() function needed
 *						by a function in SBData.c
 *
 *		Oct. 6, 1994:	Fixed bug in SBCommOpen() that caused
 * 						carriage returns (13) to be discarded.
 * 			
 *						Modified to use different method of getting
 *						serial input, and timing out read() attempts.
 *
 *						Modified to use termios, but still use
 *						ioctl calls rather than POSIX functions.
 *						POSIX functions just don't work on many
 *						systems without a lot of restrictions.
 *
 *						Added support for NEXTSTEP. Required some
 *						#ifdef statements, but it seems better than
 *						yet another version of this file.
 *
 *******************************************************************/

/** include files **/
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <string.h>

#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include <assert.h>
#include <signal.h>

#include "sbcomm.h"
#include "sbhw.h"

#ifndef SB_NEXTSTEP
/* NEXT doesn't use this for some reason */
#include <malloc.h>
#endif


/** local definitions **/
/*  size of the comm buffer  */
#define BUFFSIZE_R  0x2000	/* read buffer (8k)  */

#define MS_PER_BYTE 4 	/* time for a byte to come from the SoftBoard */

/* special stuff for NEXT ioctl calls (and maybe others?) */
#ifdef SB_NEXTSTEP
	#define TCSETA 		TIOCSETA
	#define TCSETAW   	TIOCSETAW
	#define TCGETA 		TIOCGETA
	#define TCFLSH		TIOCFLUSH
#endif

/** globals for this file **/ 
BYTE		*SBCommBuffer;	/* Receive buffer */
int			gBytesReady; 	/* number of bytes in comm buffer */
int			gBufferIndex;   /* index into receive buffer  */
int			gCommID = -1;	/* comm port dev handle */
BOOL		gErrorFlag;	    
int			gErrorValue;

struct termios gTerm;	/* global terminal mode stuct */
struct termios OldTermModes;	/* to save the old state */

/** local function prototypes **/
void SetError(int error);
void SetOpenError(int gCommID);
void TranslateErrorString(int error, char *pString); 
void DebugCommRead( BYTE *Data, int Bytes);


/* ------------------------------------------------------------------
**  Error handling note:

   ------------------------------------------------------------------
   Timeout errors during READs are NOT errors!  The timeout return
   value is returned, but the error flag is not set.
  
*  Timeout errors during WRITEs _are_ errors!
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
 ** SBCommOpen()
 
 *  PARAMETERS:
 		port_device:	The serial port device string.
		
 *  DESCRIPTION:
		Opens the serial port, sets up transmit and receive buffers, 
		baud rate, number of stop bits, etc. 
 
 *  RETURNS:
		An SBCOMM_ERROR (cast to an int). Usually this will be 
		COMM_NO_ERROR, which is zero.
 		
------------------------------------------------------------------------*/
int SBCommOpen( char *port_device )
{

	int CommError, result;
	char port[40];

	gCommID = -1;
	gErrorFlag = FALSE;
	gErrorValue = (int) COMM_NO_ERROR;

	/* SBDebugCommReads = 0; */

	/* reset the buffer's byte counter and index */
	gBytesReady = 0;
	gBufferIndex = 0;

 	/* Allocate mem for the internal buffer */
 	SBCommBuffer = (BYTE *) malloc( BUFFSIZE_R);
	if (SBCommBuffer == NULL) 
	{
		SetError( (int) COMM_ALLOC_FAILED );
		return( gErrorValue );	
	}

	/* Open the comm port */
	/* first, get the filename string right */
	if (strncmp("/dev/", port_device, 5))
		strcpy(port, "/dev/"),
		strcat(port, port_device);
	else
		strcpy(port, port_device);

	gCommID = open(port, O_RDWR);
	if(gCommID < 0)
	{
		CommError = gCommID;
		SetOpenError(CommError);
		free( (void*) SBCommBuffer);
		SBCommBuffer = NULL;
		SBTrace("SBCommOpen: Error opening comm port\n");
		return( gErrorValue);
	}

	/* Set the comm port parameters. */
	/* Some of this may be redundant, but docs weren't totally clear. 
	   The input mode bits probably can be left cleared since 
	   we are using raw mode, but we set them to ignore errors anyway. 
	   Basically, we want raw mode with no handshaking at all, and 8N1. 
	   Baud rates are set elsewhere as needed.  */
	tcgetattr(gCommID, &gTerm); /* get current settings */
	tcgetattr(gCommID, &OldTermModes); /* store a copy for later */
	gTerm.c_iflag = 0; /* clear all input mode bits */
	gTerm.c_iflag |= IGNBRK | IGNPAR; /* ignore any input "errors" */
	gTerm.c_oflag = 0; /* clear all output mode bits, and leave them cleared */
	gTerm.c_cflag = 0; /* clear all hardware control bits */
	gTerm.c_cflag |= CREAD | CS8 | B2400; /* we only need these */
	gTerm.c_lflag = 0; /* clear all line discipline mode bits */
	/* The VMIN/VTIME settings say that we just want whatever is ready to 
	   be read within the next 1/10 second. */
	gTerm.c_cc[VMIN] = 0; 	/* This many chars satisfies reads */
	gTerm.c_cc[VTIME] = 1;	/* or in this many tenths of seconds */
	result = tcsetattr(gCommID, TCSAFLUSH, &gTerm);
	if (result<0)
	{
		SetError(COMM_IOCTL_ERROR);
		SBTrace("SBCommOpen: Error setting port modes \n");
		return( gErrorValue);				
	}

	return( gErrorValue ); 
}


/*---------------------------------------------------------------------------
 ** SBCommClose()
 
 *  PARAMETERS:		None

 *  DESCRIPTION:	
		Closes the comm port, doing any cleanup needed.
 
 *  RETURNS:		
		Zero on success; non-zero on failure.

--------------------------------------------------------------------------*/
int SBCommClose()
{
	/* check for valid comm id */
	if(gCommID < 0) 
	{
		SBTrace("SBCommClose: Error closing comm port. Bad ID.\n");
		return(1);
	}

	/* restore the old port modes */
	/* not sure we need to do this since we should be the only ones
	   using the port, but we'll do it anyway. */
	tcsetattr(gCommID, TCSAFLUSH, &OldTermModes);
    
	/* close the serial comm port */
	close(gCommID);
    gCommID = -1;
	
	/* release the secondary buffer memory */
	free( (void*) SBCommBuffer);
	return(0);
}


/*---------------------------------------------------------------------------
 ** SBCommSendBreak()
 
 *  PARAMETERS:		None	

 *  DESCRIPTION:	
		Sends a standard break signal on the serial line. Drives the 
		data line low for at least 0.25 seconds.
 
 *  RETURNS:
 		
--------------------------------------------------------------------------*/
void SBCommSendBreak()
{
	if(gCommID < 0) 
	{
		SetOpenError(gCommID);
		SBTrace("SBCommSendBreak: Bad port gCommID\n\r");
		return;
	}

	gBytesReady = 0; 	 
#ifdef SB_NEXTSTEP
	ioctl( gCommID, TIOCSBRK, 1);
	usleep(250000); /* keep break on for 250 milliseconds */
	ioctl( gCommID, TIOCCBRK, 0);
#else
	ioctl( gCommID, TCSBRK, 0);
#endif

	return;
}


/*---------------------------------------------------------------------------
 ** SBCommFlush()
 
 *  PARAMETERS:		None	

 *  DESCRIPTION:	
		Flushes the comm port buffers, resets the secondary buffer, and 
		clears the operating system comm errors.
 
 *  RETURNS:
 		
--------------------------------------------------------------------------*/
void SBCommFlush()
{
	int result;

	/* just get rid of everything in the receive buffers */
	do
	{
		result = SBCommByteReadyWait(200);
	} while( result > 0);

	gBytesReady = 0;
	gBufferIndex = 0;

}


/*---------------------------------------------------------------------------
 ** SBCommSetBaudRate()
 
 *  PARAMETERS:	
 		Rate:	The baud rate to set the current serial port to.	

 *  DESCRIPTION:	
		Sets the baud rate of an open serial port. This entire routine
		is platform dependent.

		Only 3 specific baud rates are supported, because only those
		3 are used by the SoftBoard.

		NOTE: It is important that all the characters in the output 
		buffer are sent before the baud rate actually changes. Otherwise
		the download will not be dependable.
 
 *  RETURNS:
		An error value on failure, or COMM_NO_ERROR (zero) on success.
 		
--------------------------------------------------------------------------*/
int SBCommSetBaudRate(unsigned int BaudRate)
{
	int error;
	struct termios term;
	
	if(gCommID < 0) 
	{
		SetOpenError(gCommID);
		SBTrace("SBCommSetBaudRate: Bad port id\n");
		return( gErrorValue);
	}

	tcgetattr(gCommID, &term); 
	switch( BaudRate )
	{

/* unfortunately, there are a number of problems using POSIX functions
	in the real world, so we have some workarounds here. You may need 
	to make even another option for your system. So much for standards.. */
#ifdef SB_NEXTSTEP
		case 2400:
			term.c_ispeed = B2400;
			term.c_ospeed = B2400;
			break;

		case 9600:
			term.c_ispeed = B9600;
			term.c_ospeed = B9600;
			break;

		case 19200:
			term.c_ispeed = B19200;
			term.c_ospeed = B19200;
			break;
#else

		case 2400:
#if 1
			cfsetospeed(&term, B2400);
			cfsetispeed(&term, B2400);
#else
			term.c_cflag &= ~CBAUD;	/* clear the old baud rate */
			term.c_cflag |= B2400;
#endif
			break;

		case 9600:
#if 1
			cfsetospeed(&term, B9600);
			cfsetispeed(&term, B9600);
#else
			term.c_cflag &= ~CBAUD;
			term.c_cflag |= B9600;
#endif
			break;

		case 19200:
#if 1
			cfsetospeed(&term, B19200);
			cfsetispeed(&term, B19200);
#else
			term.c_cflag &= ~CBAUD;
			term.c_cflag |= B19200;
#endif
			break;
#endif

		default:
			SBTrace("Illegal baud rate selected\n");
			SetError(COMM_BAD_ARG);
			return( COMM_BAD_ARG);
			break;

	}

	/* Now, set the new baud rate, using the form that waits 
		for the output to drain. */
	error = tcsetattr(gCommID, TCSAFLUSH, &term);	
	if (error < 0)
	{
		SetError(COMM_IOCTL_ERROR);
		SBTrace("SBSetBaudRate: call to set params failed \n");
		return( gErrorValue );
	}
	else
		return( (int) COMM_NO_ERROR);
}


/*---------------------------------------------------------------------------
 ** SBCommByteReady()
 
 *  PARAMETERS:		None	

 *  DESCRIPTION:	
		Checks to see if the receive buffer contains data. If the secondary
		buffer is empty, it checks the operating sytem's comm buffer. If the
		OS comm buffer has data, all that data is read into the secondary
		buffer.
 
 *  RETURNS:
		The number of bytes ready (including the possibility of zero), or
		an SBCOMM_ERROR (cast to an int for the return).
 		
--------------------------------------------------------------------------*/
int SBCommByteReady()
{
	int count;
 
	if (gCommID < 0)
	{
		SetOpenError(gCommID);
		return( gErrorValue);
	}
			   
   	/* If there is data in our buffer, just tell how much. No need to 
	   do anything else. */
	if(gBytesReady) 
		return( gBytesReady );

	/* 	Else there is no data in our buffer, so we'll look in the
		system serial port buffer.  */
	gBufferIndex = 0;

	/* read all the data the OS comm buffer has */
	count = read(gCommID, SBCommBuffer, BUFFSIZE_R);

	/* 	check for an OS comm error  */
	if(count == -1)
	{
			SetError(COMM_READ_ERROR);
			return( gErrorValue);
	}

	/*  count represents the real amount of data 
		we got (which might be zero bytes) */
	gBytesReady = count;
	
	if( SBDebugCommReads)
		DebugCommRead(SBCommBuffer, count);
	
	return gBytesReady;
}


/*---------------------------------------------------------------------------
 ** SBCommByteReadyWait()
 
 *  PARAMETERS:		
 		Milliseconds:	Milliseconds to wait.	

 *  DESCRIPTION:	
		Waits up to the number of milliseconds specified in
		the passed parameter form some data to appear at the serial port.

 *  RETURNS:
		The number of bytes ready, or an SBCOMM_ERROR (cast to an int 
		for the return). COMM_TIMEOUT is a likely return value,	but it 
		does NOT set the error flag. A time out on a read operation 
		is not considered an error.
 		
--------------------------------------------------------------------------*/
int SBCommByteReadyWait( int Milliseconds)
{
	int count, tenths;
	int result;

	if (gCommID < 0)
	{
		SetOpenError(gCommID);
		return( gErrorValue);
	}
		
	/* If there is data in our buffer, just tell how much. No need to 
	   do anything else. */
	if(gBytesReady) 
		return( gBytesReady );

	/* 	Else there is no data in our buffer, so we'll look in the
		system serial port buffer.  */
	gBufferIndex = 0;
	
	/* set up the port to wait the specified time for some data */
	tenths = Milliseconds/100;
	if(tenths == 0 )
		tenths = 1; 

	/* Since the tty driver is set to return after 1/10 second if there
	   is no data, we can count the tries to know how _about_ long we have
	   tried to get data. This will not be very accurate if the OS
	   takes much time during each loop. On many systems, this loop 
	   could be modified to use a more accurate timer, such as the 
	   setitimer() function, or the ftime() function, which returns
	   a struct with a millisecond time field. */
	do
	{
		/* 	Read all the data the OS comm buffer has. This will return 
			after 1/10 second if no data is present. */
	  count = read(gCommID, SBCommBuffer, BUFFSIZE_R); 
	  if(count)
	    break;
	  tenths--;
	  /* let the OS have some time */
	  if( SBReleaseControl() == TRUE)
	    return(COMM_USER_ABORT);
	  
	}while( (count == 0) && (tenths > 0) );

	/* 	check for an OS comm error  */
	if(count == -1)
	  {
		SetError(COMM_READ_ERROR);
		return( gErrorValue);
	}

	/*  count represents the real amount of data we got, 
	 	or the fact that we timed out waiting for data */
	gBytesReady = count;

	if (count == 0)
	{
		return( (int) COMM_TIMEOUT );
	}
	
	if( SBDebugCommReads)
		DebugCommRead(SBCommBuffer, count);
	
	return gBytesReady;
}


/*---------------------------------------------------------------------------
** SBCommPeekMFG()
 
 *  PARAMETERS:		None

 *  DESCRIPTION:	
		Special function that peeks ahead in the buffer to look for the 
		string "MFG-". This is needed to help detect the case of the
		SoftBoard data handling routine getting lost because of some
		data from SoftBoard getting dropped while a user is writing.
	
 *  RETURNS:
		TRUE if it finds "MFG-" in the next four characters, otherwise
		returns FALSE.
 		
--------------------------------------------------------------------------*/
BOOL SBCommPeekMFG()
{
	BYTE *data;
	BYTE DriverData[4];
	int DriverBytesNeeded, count;

	/* If we are going to get this data, it will be part of a message 
	   that has already been partially received. Therefore, worst case
	   wait for the next four bytes will about MS_PER_BYTE * 4. Then
	   we'll throw in some extra time for good measure. We have plenty of
	   time because this should never happen except during debugging.*/
	SBWait( MS_PER_BYTE * 4 + 200); 

	if(gBytesReady < 4)
	{	      
		/* how many bytes _might_ still be in the OS buffer? */
		DriverBytesNeeded = 4 - gBytesReady;


		/* 	Try to read the number of bytes we need. This will return 
			within 1/10 second, and we may or may not have the number 
			of bytes expected. */
		count = read(gCommID, DriverData, DriverBytesNeeded);
						
		if(count < 0)
		{
				SetError(COMM_READ_ERROR);
				return( gErrorValue);
		}
		else if (count > 0)
		{
			if( SBDebugCommReads)
				DebugCommRead(DriverData, count);

			/* Copy the new data into our buffer, almost like SBCommByteReady
			   does, except we have to splice it onto existing data.*/
			memcpy( &SBCommBuffer[gBufferIndex+gBytesReady], DriverData, count);
			gBytesReady += count; /* now we'll have the new data too */
		}
	}

	if (gBytesReady >= 4)
	{
		data = &SBCommBuffer[gBufferIndex];
		if( (data[0] == 'M') &&  (data[1] == 'F') &&
			(data[2] == 'G') && (data[3] == '-') )
		{
			return( TRUE );	
		}
		else
			return( FALSE );
	}
	else
		return( FALSE );

	
}


/*---------------------------------------------------------------------------
 ** SBCommReadByte()
 
 *  PARAMETERS:		None

 *  DESCRIPTION:	
 		Reads a byte from the comm buffer. If there is a byte in either
		the operating system receive buffer or the secondary buffer, it
		will be returned.
	
 *  RETURNS:
		The byte from the comm port on success, or an SBCOMM_ERROR value 
		(cast to an int).
 		
--------------------------------------------------------------------------*/
int SBCommReadByte()
{
	int value;

	if (gCommID < 0)
	{
		SetOpenError(gCommID);
		return( (int)gErrorValue );
	}

	value = SBCommByteReady();
	if(value >= 1)
	{
		gBytesReady--;
		return SBCommBuffer[gBufferIndex++];
	}
	else if (value = 0)
	{
		return( (int) COMM_NO_DATA);
	}
    else
    {
    	/* comm error (was set in SBCommByteReady() */
		return( (int)gErrorValue ); 
	}
}


/*---------------------------------------------------------------------------
 ** SBCommReadData()
 
 *  PARAMETERS:	
 		Buffer:			Buffer to copy the data to.
 		BytesRequested:	Number of bytes asked for.	

 *  DESCRIPTION:
		Gets the number of bytes requested from the serial port, 
		if there is enough data waiting.
 
 *  RETURNS:
 		Returns the number of bytes copied to the buffer, or COMM_NO_DATA 
 		or an SBCOMM_ERROR (cast to an int for return). Never returns 
 		more than the number of bytes requested, but may return less.	
		
--------------------------------------------------------------------------*/
int SBCommReadData(BYTE *Buffer, int BytesRequested)
{
	int BytesReady;
	int BytesToRead;

	if(gCommID < 0) 
	{
		SetOpenError(gCommID);
		return( (int)gErrorValue);
	}

	/* We will never read more than SBCommByteReady() returned. */
	BytesReady = SBCommByteReady();
	if(BytesReady > 0)
	{
		if( BytesRequested < BytesReady  )
			BytesToRead = BytesRequested;
		else
			BytesToRead = BytesReady;

		memcpy( Buffer, &SBCommBuffer[gBufferIndex], BytesToRead );
		gBytesReady = gBytesReady - BytesToRead;
		if(gBytesReady == 0)
			gBufferIndex = 0;		
		else
			gBufferIndex = gBufferIndex + BytesToRead;

		return BytesToRead;
	}
	else
		return(BytesReady);	/* error or no data */
}


/*---------------------------------------------------------------------------
 ** SBCommReadDataWait()
 
 *  PARAMETERS:	
 		Buffer:			Buffer to copy the data to.
 		BytesRequested:	Number of bytes asked for. Must be 256 or less.	

 *  DESCRIPTION:
		Gets number of bytes requested from the serial port, if the data
		is available within the allotted time, and copies them to the 
		passed buffer address. The time to wait for the data is dependent 
		on the amount of time required for the SoftBoard to send the 
		number of bytes requested. If ByteRequested is not available
		withihn the alloted time, any bytes received to that point are 
		added to the SBComm queue. Unless the full amount of data 
		requested is received, no data is returned.

		This function is designed to be used to get small amounts of 
		data. This keeps the time it will wait down to a reasonable
		level.
 
 *  RETURNS:
 	  	The number of bytes copied to the buffer, or an SBCOMM_ERROR 
		(cast to an int	for the return). COMM_TIMEOUT is a likely return 
		value,	but it 	does NOT set the error flag. A time out on a 
		read operation is not considered an error. Never returns more 
		or less data than requested.

--------------------------------------------------------------------------*/
int SBCommReadDataWait(BYTE *Buffer, int BytesRequested)
{
	unsigned DriverBytesNeeded;
	int result, count, tenths;
	BYTE DriverData[256];
	
	if(gCommID < 0) 
	{
		SetOpenError(gCommID);
		return( (int)gErrorValue);
	}

	if(BytesRequested == 0 )
		return(0);
 
	/* 	make sure they aren't asking for too much data, which 
		might cause us to overrun our internal buffer */
	if(BytesRequested > 256)
	{
		SetError( COMM_BAD_ARG );
		return( (int) COMM_BAD_ARG);
    }

	/* see if we need any data from the OS comm driver */
	if( gBytesReady >= BytesRequested)
	{
		/* we already have plenty of data, so just send it */
		memcpy( Buffer, &SBCommBuffer[gBufferIndex], BytesRequested );
		gBytesReady = gBytesReady - BytesRequested;
		if(gBytesReady == 0)
			gBufferIndex = 0;
		else
			gBufferIndex = gBufferIndex + BytesRequested;
		return( BytesRequested);
	}
	else
		DriverBytesNeeded = BytesRequested - gBytesReady;


	/* number of 1/10 second periods to wait */
	tenths = (BytesRequested * MS_PER_BYTE)/100 + 2;

	/* 	Try to read the number of bytes we need.  This may return with
		less data than we need, so we have to loop until we either 
		get the data we need or we use the time allowed. This will not 
		be very accurate if the OS takes much time during each loop. 
		
		On many systems, this loop could be modified to use a more 
		accurate timer, such as the setitimer() function, or the 
		ftime() function, which returns a struct with a millisecond
		time field. */
	count = 0;
	do
	{
		result = read(gCommID, &DriverData[count], DriverBytesNeeded - count);
		if (result < 0)
			break;

		if (result > 0)
		{
			count += result;
			if (count == DriverBytesNeeded)
				break;
		}

		tenths--;

		/* let the OS have some time */
		if( SBReleaseControl() == TRUE)
			return(COMM_USER_ABORT);

	}while( (tenths > 0) && (count < DriverBytesNeeded) );

	/* 	check for an OS comm error  */
	if(result == -1)
	{
		SetError(COMM_READ_ERROR);
		SBTrace("SBCommByteReadyWait: Error reading driver data \n");
		return( gErrorValue);				
	}

	/* If we got all the bytes we needed, then copy them and the 
	   data we already had into the buffer to be returned */
	if (count == DriverBytesNeeded )
	{
		if( SBDebugCommReads)
			DebugCommRead(DriverData, count);
		
		/* copy the existing data into our buffer to be returned */
		memcpy( Buffer, &SBCommBuffer[gBufferIndex], gBytesReady );
		
		/* copy the driver data just after the SBCommBuffer data */	
		memcpy( Buffer+gBytesReady, DriverData, count );
		
		/* we read all the data, so reset these */
		gBytesReady = 0;	
		gBufferIndex = 0;
	
		return(BytesRequested);
	}

	/* if there was some data returned, but not as much 
	   as we requested, then we need to copy it into our
	   secondary buffer, and not pass it back to the calling
	   routine */
	if ( (count > 0) && (count < DriverBytesNeeded) )
	{
		memcpy( &SBCommBuffer[gBufferIndex+gBytesReady], DriverData, count);
		gBytesReady += count; /* now we'll have the new data too */
	}

	/* if we didn't return above with all the data requested, then 
	   we timed out */	
	return((int) COMM_TIMEOUT);
		
}

 
/*---------------------------------------------------------------------------
 ** SBCommWriteByte()
 
 *  PARAMETERS:	
 		ByteToSend: The byte to write to the serial port.	

 *  DESCRIPTION:
 		Sends a single byte to the serial port. 
 
 *  RETURNS:
		Returns COMM_NO_ERROR if good write. Returns COMM_WRITE_ERROR
		if there was a comm error.  
 		
--------------------------------------------------------------------------*/
int SBCommWriteByte(BYTE ByteToSend)
{

	int result;
	BYTE byte = ByteToSend; /* need a local var */
    
	if(gCommID < 0) 
	{
		SetOpenError(gCommID);
		return( (int)gErrorValue);
	}

	result = write(gCommID, &byte, 1);

	if(result != 1)
	{
		SetError(COMM_WRITE_ERROR);
		return((int) gErrorValue);
	}
	else
	{
		return( (int)COMM_NO_ERROR); 
	}
	
}


/*---------------------------------------------------------------------------
 ** SBCommWriteData()
 
 *  PARAMETERS:	
 		Buffer: Pointer to the buffer containing the data to write.	
		NumBytes: The number of bytes to write.

 *  DESCRIPTION:
 		Sends a block of data bytes to the serial port. .
 		
 		The data will not necessarily go out the serial port before
 		the function returns, but the data will be copied to the 
 		transmit buffer if there is not an error.

		NOTE: This routine is designed to send relatively small blocks
		of data. With 256 byte blocks, there should rarely be a 
		error returned. This routine has been changed a bit from
		the Windows version because of the inability to find out how
		much room is left in the transmit buffer. For most purposes, 
		however, this should not be a problem.
		
 *  RETURNS:
		Returns COMM_NO_ERROR if good write. Returns one of the 
		error enums if there was a comm error.  
		
--------------------------------------------------------------------------*/
int SBCommWriteData(BYTE* Buffer, int NumBytes)
{
	int result;
    
	if(gCommID < 0) 
	{
		SetOpenError(gCommID);
		return( (int)gErrorValue);
	}

	result = write(gCommID, Buffer, NumBytes);

	if(result != NumBytes)
	{
		SetError(COMM_WRITE_ERROR);
		return((int) gErrorValue);
	}
	else
	{
		return( (int)COMM_NO_ERROR); 
	}
	
}


/*---------------------------------------------------------------------------
 ** SBCommIsError()
 
 *  PARAMETERS:		None

 *  DESCRIPTION:	
		Checks to see if the comm error flag has been set.
 
 *  RETURNS:
		TRUE(1) if an error has happened, FALSE(0) otherwise.
 		
--------------------------------------------------------------------------*/
BOOL SBCommIsError()
{
	if(gErrorFlag)
		return(TRUE);
	else
		return(FALSE);
}


/*---------------------------------------------------------------------------
 ** SBCommGetgErrorValue()
 
 *  PARAMETERS:		None		

 *  DESCRIPTION:	
 		
 *  RETURNS:
		Returns the SCOMM_ERROR value of the current error.

--------------------------------------------------------------------------*/
int SBCommGetgErrorValue()
{
	return( gErrorValue );
}


/*---------------------------------------------------------------------------
 ** SBCommGetErrorString()
 
 *  PARAMETERS:		
		Buffer: A pointer to a buffer to copy the error string into. The
		buffer should have space for a string the length of 
		SBCOMM_ER_STR_MAX.

 *  DESCRIPTION:	
 		Gets a string that describes the current error. The string can be 
 		used for displaying the exact error to the user.

 *  RETURNS:
		Copies the error string into the passed buffer. No function
		return.
 		
--------------------------------------------------------------------------*/
void SBCommGetErrorString( char* Buffer)
{
	TranslateErrorString( gErrorValue, Buffer );
}


/*---------------------------------------------------------------------------
 ** SBCommClearError()
 
 *  PARAMETERS:		None		

 *  DESCRIPTION:	
		Clears the current error.
 
 *  RETURNS:		Nothing
 		
--------------------------------------------------------------------------*/
void SBCommClearError()
{
	gErrorFlag=FALSE;
	gErrorValue = (int) COMM_NO_ERROR;
}



/***** Local error helper routines *****/ 

/*---------------------------------------------------------------------------
 ** SetError()
 
 *  PARAMETERS:		
		Error:	The error value to be entered as the SBComm error.

 *  DESCRIPTION:	
 		Takes an error number, which might be from the operating system
		comm routines or passed as a constant, and sets the global SBComm
		error value and error flag.

 *  RETURNS:	Nothing
 		
--------------------------------------------------------------------------*/
void SetError(int Error)
{
	char err_string[SBCOMM_ERR_STR_MAX];

	if (Error != (int) COMM_NO_ERROR)
	{
		gErrorFlag = TRUE;
		gErrorValue = Error;
		SBCommGetErrorString( err_string );
		SBTrace( err_string );
		SBTrace( "\n" );
	}
}


/*---------------------------------------------------------------------------
 ** SetOpenError()
 
 *  PARAMETERS:
 		Error:	The error from the OS open() command.		

 *  DESCRIPTION:	
 		Takes an error returned from the OS open() function,
 		and sets the appropriate SBCommError type. 

		NOTE: this version does not use the passed Error, but
		we'll leave it in for now in case we want to use it later.

 *  RETURNS:	Nothing
 		
--------------------------------------------------------------------------*/
void SetOpenError(int Error)
{
	char err_string[SBCOMM_ERR_STR_MAX];

	gErrorFlag = TRUE;
	gErrorValue = (int) COMM_UNKNOWN_OPEN;

	SBCommGetErrorString( err_string );
	SBTrace( err_string );
	SBTrace( "\n" );
}

/*---------------------------------------------------------------------------
 ** TranslateErrorString()
 
 *  PARAMETERS:
 		Error: 		SBCOMM_ERROR value to be translated.
 		pString:	buffer to copy the string to.		

 *  DESCRIPTION:	
 		Takes an error value and copies a descriptive string to the 
		passed buffer.

 *  RETURNS:
		No return value. Copies the error string into the passed buffer.
 		
--------------------------------------------------------------------------*/
void TranslateErrorString( int Error, char *pString)
{
	switch(Error)
	{
		case COMM_NO_ERROR:
			strcpy(pString, "No error detected");
			break;
		case COMM_TIMEOUT:
			strcpy(pString, "Time-out");
			break;
		case COMM_USER_ABORT:
			strcpy(pString, "User aborted comm operation");
			break;
		case COMM_NO_DATA:
			strcpy(pString, "No data read");
			break;
		case COMM_ALLOC_FAILED:
			strcpy(pString, "Error allocating memory");
			break;
		case COMM_UNKNOWN_OPEN:
			strcpy(pString, "Unknown comm open error.");
			break;
		case COMM_BAD_ARG:
			strcpy(pString, "Bad argument passed to SBComm function.");
			break;
		case COMM_IOCTL_ERROR:
			strcpy(pString, "Error from ioctl() call to set port params");
			break;
		case COMM_READ_ERROR:
			strcpy(pString, "Error reading from the serial port");
			break;			
		case COMM_WRITE_ERROR:
			strcpy(pString, "Error writing to the serial port");
			break;			
		case COMM_CLOSE_ERROR:
			strcpy(pString, "Error closing the serial port");
			break;			
		default:
			strcpy(pString, "Unknown comm error.");
			break;
	}
}


/* send the received chars out to the debug device */
void DebugCommRead( BYTE *Data, int Bytes)
{
	int i, j;
	static char buf[85];
	 
	for (i=0, j=0;i<Bytes;i++)
	{
		sprintf( &buf[j],"%03d ", Data[i]);
		j += 4;
		if( j > 79 )
		{
			buf[j] = 0; /* null terminate the string */
			SBTrace( buf );
			j = 0;
		}
	}

	if(j)  
	{
		buf[j] = 0; /* null terminate the string */
		SBTrace( buf );
	}
}


