/*******************************************************************
 *
 *  FILE:			sbcomm.h 				
 *
 *  DESCRIPTION:	Defines the standard serial port (comm port)
 *					functions for the SoftBoard sample code. These 
 *					functions use a separate buffer (SBCommBuffer) 
 * 					for the recieved data than the operating system's 
 * 					serial port buffer .
 *
 *	FUNCTIONS:		SBCommOpen( Port ): Opens a serial port, using
 * 					the operating system's facilities. The Port
 * 					parameter indicates a system port number (this
 * 					may need to be a different type, and additional
 * 					parameters may be needed on some systems). Most of
 *					the serial port options are fixed by the SoftBoard,
 *					so not many options are expected to be passed to
 *					this routine. 
 *
 *					SBCommSendBreak(): Send a break signal.
 *
 *					SBCommFlush(): Flushes the serial port buffers, and 
 *					resets SBCommBuffer.
 *
 *					SBCommSetBaudRate( BaudRate ): Sets the baud rate for
 *					the open comm port. It is necessary to change the
 *					baud rate during various stages of the download,
 *					so this function must be present.
 *
 *					SBCommClose(): Closes the comm port opened with
 *					the SBCommOpen() function.
 *
 *					SBCommByteReady(): Checks to see if there is data
 *					ready in the SBCommBuffer buffer. Forces a call to 
 * 					get any data in the operating system's serial buffer 
 * 					if no data is waiting in SBCommBuffer. 
 *
 *					SBCommByteReadyWait( ms ) - Same as SBCommByteReady,
 *					except waits for the requested number of milliseconds
 *					before returning if no data is available.
 *
 *					SBCommPeekMFG(): Special function that peeks ahead in
 *					to look for the string "MFG-". This is needed to help 
 *					detect the case of the SoftBoard data handling routine 
 *					getting lost because of some data from the SoftBoard 
 *					getting dropped while a user is writing. 
 *
 *					SBCommReadByte(): Returns a byte from the serial port
 *					if there is one waiting.
 *
 *					SBCommReadData( pData, NumBytes ): Gets Numbytes requested 
 *					from the serial port if there is enough data waiting.
 * 					Never returns more than the number of bytes requested, 
 * 					but may return less.
 *
 *					SBCommReadDataWait( pData, NumBytes): Gets Numbytes 
 * 					requested from the serial port if that amount of data is
 * 					available within a reasonable amount of time. The time
 *					to wait for the data is dependent on the amount of time
 *					required for the SoftBoard to send that number of bytes
 *					requested. Fails if the requested number of bytes is not 
 *					available in the time allotted.
 *
 *					SBCommWriteByte(): Sends a byte to the serial port. 
 *
 *					SBCommWriteData( pData, NumBytes ): Writes NumBytes bytes
 * 					to the serial port. Never writes a partial block.  
 * 					
 *					SBCommIsError(): Checks to see if a comm error has
 *					occurred. Returns TRUE or FALSE.
 * 
 *					SBCommGetErrorValue(): Returns a SBCOMMERR value 
 *					to identify the error.
 *
 *					SBCommGetErrorString(): Returns the error string that
 *					corresponds to the current comm error condition.
 *					
 *					SBCommClearError(): Clears a comm error.
 *									
 *
 *    HISTORY:    	Sept. 7, 1994:	First UNIX release - Guy Townsend

					Sept. 22, 1994:	Added COMM_CLOSE_ERROR. - GAT
 *
 *******************************************************************/
#if !defined( _SBCOMM_H_ )
#define _SBCOMM_H_

#include "sbhw.h"

/*	Error values for the comm functions. Often used as int values. */
enum SBCOMM_ERROR { 
				COMM_NO_ERROR =				0,
				COMM_TIMEOUT =				-1,
				COMM_USER_ABORT =			-2,
				COMM_NO_DATA = 				-3,
				COMM_ALLOC_FAILED =			-99,
				COMM_UNKNOWN_OPEN =			-108,
				COMM_BAD_ARG =				-200,
				COMM_IOCTL_ERROR =			-201,
				COMM_READ_ERROR =			-202,
				COMM_WRITE_ERROR = 			-203,
				COMM_CLOSE_ERROR = 			-204
};

/* Max length returned from SBCommGetErrorString */
#define SBCOMM_ERR_STR_MAX 80 

/* prototypes for routines in sbcomm.c */
int SBCommOpen( char *port_device); 
int SBCommClose();
void SBCommSendBreak();
void SBCommFlush();
int SBCommSetBaudRate( unsigned int BaudRate ); 
int SBCommByteReady();
int SBCommByteReadyWait( int ms );
BOOL SBCommPeekMFG();
int SBCommReadByte();
int SBCommReadData( BYTE* pData, int NumBytes );
int SBCommReadDataWait( BYTE* pData, int NumBytes );
int SBCommWriteByte(BYTE c);
int SBCommWriteData( BYTE* pData, int NumBytes );
BOOL SBCommIsError();
int SBCommGetErrorValue(); /* */
void SBCommGetErrorString( char* pString);
void SBCommClearError(); 

#endif /* _SBCOMM_H_ */


