/*************************************************************************
 *
 *    FILE:			sbdata.c
 *
 *    DESCRIPTION:	Routines to handle the data stream to and from
 *					the SoftBoard. As messages from the board are
 *					received, appropriate functions in sbmsg.c are
 *					called to decipher the data and take actions 
 *					as needed. All errors in the SoftBoard data
 *					stream are caught in this file.
 *
 *					NOTE: You may want to call your own error handling
 *					routines from the error handling routines in this 
 *					fies that handle serial port errors.
 *
 *    HISTORY:    	
 *		Sept. 8, 1994:	First Release - Guy Townsend / Microfield
 * 
 *		Sept. 22, 1994:	Added conditional compile header include
 *						for NEXTSTEP.
 *
 *						Also changed error handling flow so we can't
 *						get stuck in the case of a data error.
 *
 *		Oct. 6, 1994:	Added SBStatus.Recording field updating to  
 *						HandleSBMessage() when receiving the messages 
 *						from the SoftBoard relating to starting and
 *						stopping recording mode.
 *
 *						Added some SBTrace statements before calling
 *						HandleCommError() to aid debugging.
 *
 *	 					Added code to SBDataIsDownloadDone() to avoid
 *						a possible re-entrancy problem. 
 *
 *		Dec. 19, 1994:	Fixed error in ResyncDatastream(), which caused
 *						an infinite loop if a fatal error occurred.
 * 
 *		May  8, 1995:	Changed ResycDatastream() not to ask for status
 *						information if in Record mode, since it should 
 *						be coming anyway.
 *
 *		June 5, 1995:	Changed SendCommand to send the bytes as a block
 *						to be compatible with NT.
 * 
 *		June 13, 1995:	Added functionality to return from CheckPort
 *						after 150ms, even if there is more data; but only
 *						if compiled for Windows. Needed to support the 
 *						devkit DLL, which has the app poll for data. 
 *						
 *						Changed the CheckPort function not to wait around
 *						more characters coming in the port, but return
 *						the first time no more characters are ready.
 *						
 *						Changed some time values to wait for data from
 *						500 to 100 ms.
 *
 *		July 28, 1995:	Made all comm errors fatal. No re-synch of the 
 *						datastream is done anymore.
 *
 *		Oct. 11, 1995:	Added handling of TAG_TRACKING_RESUMED.
 *		
 ************************************************************************/

/** include files **/
#include <assert.h>
#include <stdio.h>
#include <string.h>

#include "sbhw.h"
#include "sbout.h"
#include "sbdata.h"
#include "sbcomm.h"
#include "sbtags.h"
#include "sbmsg.h" 

/** global flags **/
BOOL SB_DeltaDrawMode = FALSE;

/** internal functions **/
void HandleCommError();
void GetSBData();
void HandleSBMessage();
void ShowBadTag();
void ShowCommandSent( short Tag, BYTE *Data);

/*--------------------------------------------------------------------------
 ** SBDataSendCommand
 
 *  PARAMETERS:
 		Tag:		Command to send to the SoftBoard. Defined in sbtags.h
 		Data:		Pointer to the data to be sent. 		

 *  DESCRIPTION:	
 		The sole routine to send commands to the board. The tag of the
 		command sent, preceded by the attention byte. Only one command
 		currently takes data, and is handled as a special case. 
 		
 		Note that the Tag field here is defined as a byte, whereas
 		a Tag in the sections of code that receive data from the 
 		SoftBoard is defined as a short. This is because Tags used 
 		to send commands to the board are limited to a single 8-bit
 		value, whereas Tags coming from the board may be a 16-bit 
 		value (although none currently are more than 8 bits).
 		
 		CAUTION: This routine may need to change in the future, as there 
 		are plans to change the protocol to include a response from the 
 		board before sending the tag and data. All the changes will
 		probably be able to be handled transparently to the rest of 
 		the application by using this routine to contain any specifics 
 		regarding sending commands to the SoftBoard. At a minimum, keeping
 		all sending of commands to SoftBoard in one routine will make
 		future changes much easier.
 
 *  RETURNS:	Nothing
 		
--------------------------------------------------------------------------*/
void SBDataSendCommand( BYTE Tag, BYTE *Data)
{
	int result;
	BYTE buf[5];

	/* For lack of a better place to put these ... */
	/* They will normally go at the top application level,
		but this code is supposed to be system independent.
		At least they will be sure to get called here, even 
		though it will be after the download (which avoids
		having the DSP binary code show on the debug output).
		These variables are defined in sbhw.h, and will be
		set to zero (FALSE) by the compiler because they 
		are global. */
	
	/* SBShowDebugOutput = TRUE; */
	/* SBDebugCommReads = TRUE; */

	/* send the tag byte */
	SBWait(100);

	buf[0] = 0xFF;
	buf[1] = Tag;
	result = SBCommWriteData( buf, 2 );
	if( result != COMM_NO_ERROR)
	{
		SB_ShowError("Error writing to comm port");
		return;
	}
	
	/* currently the only command that sends data */
	if(Tag == TAG_SET_STANDBY_IDLE_TIME)
	{
		assert(Data != NULL);
		SBWait(100);  /* need this pause before sending data */
		result = SBCommWriteByte( Data[0] );
		if( result != COMM_NO_ERROR)
		{
			SB_ShowError("Error writing to comm port");
			return;
		}
			
		SBWait(100);		
		result = SBCommWriteByte( Data[1] );
				if( result != COMM_NO_ERROR)
		{
			SB_ShowError("Error writing to comm port");
			return;
		}

	}
	
	/* just for inital development */
	ShowCommandSent( Tag, Data);
	
}


/*---------------------------------------------------------------------------
 ** SBDataCheckPort
 
 *  PARAMETERS:		None

 *  DESCRIPTION:	       
 		This routine is called on a regular basis to check the serial port
 		for data from the SoftBoard. All handling of messages from the 
 		SoftBoard is handled through this routine.
 		
 		The main application needs to set a way to call this routine
 		regularly. The exact method is up to the developer. Setting a 
 		timer can be used under Windows, for instance. A good time between
 		calls might be 200 milliseconds or so. This routine reads all the 
 		data available from the serial port without returning, so when it
 		does return, it was only because there was a pause in the data 
 		being sent.
 		
 		There are #ifdef statments checking for whether this is a MSWindows
 		build. If it is, we don't stay too long, but leave after about 100 ms, 
 		even if there is more data available. This is because the DLL produced
 		for the Devkit waits for the application to poll for data, and needs
 		to return with the data quickly.
 		
 		This routine protects itself against re-entrancy, so it may be 
 		called repeatedly from the timer without harm.
 
 *  RETURNS:	Nothing
 		
--------------------------------------------------------------------------*/
void SBDataCheckPort()
{

	#ifdef _MSWINDOWS_
	DWORD TimeUp;
	#endif

	int result;	
	
	/* 	Make sure the download of the DSP code has been done successfully,
		or we can't	get any data from the SoftBoard */
	if( !SBStatus.DownloadDone)
		return;
	
	/* stop re-entrancy */
	if( SBData.Busy )
		return;
	else
		SBData.Busy = TRUE;

	/* check the port to see if SoftBoard sent any data */
	result = SBCommByteReady();
	if(  result == 0 )
		; /* no data waiting, do nothing */	
	else if( result < 0)
	{
		SBTrace("SBDataCheckPort(), SBCommByteReady error/n");	
		HandleCommError(); /* had a comm error */
	}
	else /* we have some data waiting */
	{
		#ifdef _MSWINDOWS_
		TimeUp = GetTickCount() + 100; /* return after 100 ms */
		#endif
		
		do  /* get all the SoftBoard messages waiting */
		{
			GetSBData(); /* get a message from the SoftBoard and handle it */
			
			#ifdef _MSWINDOWS_
			/* If our time is up for this trip, then go home until we're
				called again. */
			if( GetTickCount() > TimeUp)
				break;
			#endif
				
			result = SBCommByteReady(); /* see if there is any more coming */
			if(result < 0)
			{
				if(result != COMM_TIMEOUT) /* timeout is ok */
				{
					SBTrace("SBDataCheckPort(), ByteReadyWait() error\n");
					HandleCommError();
				}

				break; /* no more data */
			}

		}while( result > 0); /* quit when no more data, or comm error */
    }
	SBData.Busy = FALSE; /* Don't leave without resetting this! */		
}


/*---------------------------------------------------------------------------
 ** HandleCommError
 
 *  PARAMETERS:		None
 
 *  DESCRIPTION:	
 		Handles serial comm errors resulting from a call to one of the 
 		SBCommXXX routines. This routine is called if an incomplete message
 		is received from the SoftBoard, or if a hardware error occurs 
 		when reading or writing the serial port.
 		
 		This routine stops communicating with the SoftBoard and calls
 		the OnFatalCommError() routine with the errror string from
 		the SBComm routines.
 		
 *  RETURNS:	Nothing.
 		
--------------------------------------------------------------------------*/
void HandleCommError()
{
	char buf[SBCOMM_ERR_STR_MAX];
	
	SB_DeltaDrawMode = FALSE; /* can't be in DD mode if an error */
	SBStatus.DownloadDone = FALSE; /* we won't try to talk to the board */
	
	SBTrace("In HandleCommError()" );
	SBCommGetErrorString(buf);
	SB_ShowError(buf);
	SBCommClearError();
	/* send back the error string when notifying of comm error */
	OnFatalCommError(buf);

}


/*---------------------------------------------------------------------------
 ** GetSBData
 
 *  PARAMETERS:		None.
 
 *  DESCRIPTION:	
		Actually decodes the data stream from the board, and assembles it
		into the SBData structure for use by other routines. 		
 		
 *  RETURNS:	Nothing. 
 		
--------------------------------------------------------------------------*/
void GetSBData()
{
	BYTE byte1, byte2;
	BYTE temp[4];
	int result;
	static char buf[SB_MAX_STR_LEN];
	
	result = SBCommReadByte();
	if( result < 0)
	{
		SBTrace("SBData() ReadByte 1 error\n");
		HandleCommError();
		return;
	}
	else
		byte1 = (BYTE) result;

	/* 	if we are in delta draw mode, we have to handle the 
		first two bytes as a special case */
	if( SB_DeltaDrawMode )
	{
		/* check for escape signal */
		if( byte1 == 0x80 )
		{
			SB_DeltaDrawMode = FALSE;
			OnLeaveDeltaDrawMode();
			return;
		}
		else
		{
			SBCommByteReadyWait(100); /* 2nd byte needs time to arrive */
			result = SBCommReadByte();
			if( result < 0)
			{
				SBTrace("SBData() ReadByte 2 error\n");
				HandleCommError();
				return;
			}
			else 
				byte2 = (BYTE) result;
		}

		/* If characters are being missed, and the escape signal
			gets missed, we might end up stuck in delta draw mode
			and interpretting every pair of bytes received as
			delta draw numbers.	So, we look for the status
			messages that will come regularly in record mode.
			There are a total of 6 bytes that have match exactly
			in order for us to decide that we really have a 
			status message, and not 3 delta draws. */
 		if( (byte1 == 0xC9) && (byte2 == TAG_SOFTBOARD_STATUS) )
		{
			if ( SBCommPeekMFG() )
			{
				SB_DeltaDrawMode = FALSE;
				OnLeaveDeltaDrawMode();
				SB_ShowError("Serial Data Lost. Missed Delta Draw Escape.");
				HandleCommError(); /* must have had a comm problem */
				//ResyncDatastream();
				return;	
			}
			else
			{
				/* It was just coincidence that the first 2 bytes matched
				the status message control byte and tag, or there was a 
				comm error. So either handle the error, or go on as if
				nothing has happened. */
				if (SBCommIsError())
				{
					SBTrace("SBData() IsCommError returned error\n");
					HandleCommError();
					return;
				}
			}
		}
		
		/* make a tag so we can treat it like other messages */
		SBData.Tag = TAG_DELTA_DRAW;
        /* make it look almost like a regular DRAW message*/
        SBData.DataSize = 2;
		SBData.Data[0] = byte1;
		SBData.Data[1] = byte2;

		/* now handle the delta info as a regular message */
		HandleSBMessage();

	}	/* end delta draw processing */
	else
	{
	/* decode control byte */
		/* check for 110x in high nibble */
		/* if not, then we're lost,  and must have had some kind of comm error */
		if( (byte1 & 0xE0) != 0xC0 )
		{
			SBTrace("Invalid control byte\n");
			// ResyncDatastream();
			HandleCommError();
			return;
		}
		/* check for Extended Tag */
		/* if extended, get four more bytes and set the tag and datasize */
		if( byte1 & 0x10)
		{
			SBCommByteReadyWait(100);
			result = SBCommReadDataWait( temp, 4);
			if( result == 4)
			{
				SBData.DataSize = (short)temp[0]; /* fill low byte */
				SBData.DataSize &= (short)temp[1] << 8; /* fill high byte */

				SBData.Tag = (short)temp[2]; /* fill low byte */
				SBData.Tag &= (short)temp[3] << 8; /* fill high byte */
			}
			else
			{
				SBTrace("SBData() ReadReadDataWait 1 error\n");
				HandleCommError();
				return;
			}
		}
		else
		{
			/* 	else if not extended store data size from  
				lower nibble, and then get tag byte */
			SBData.DataSize = (short) (byte1 & 0x0F); 
			SBCommByteReadyWait(100);
			result = SBCommReadDataWait(temp,1);
			if( result == 1)
				SBData.Tag = temp[0];
			else
			{
				SBTrace("SBData() ReadDataWait 2 error\n");
				HandleCommError();
				return;
			}
        }
		
		/* 	make sure data size value is realistic, if not
			we must have had a comm error, or we're out of sync */		
		if( SBData.DataSize > MAX_SBDATA )
		{
			HandleCommError();
			return;
		}
		
		/* get data bytes, using datasize */
		SBCommByteReadyWait(100);
		result = SBCommReadDataWait( SBData.Data, SBData.DataSize );
		if( result < 0)
		{
			SBTrace("SBData() ReadDataWait 3 error\n");
			sprintf(buf, " Expected %d bytes, received %d result", 
					SBData.DataSize, result);
			SBTrace(buf);
			HandleCommError();
			return;
		}
		
		if( result != SBData.DataSize )
		{
			sprintf(buf,"Couldn't get all data for tag: %d \n Expected %d bytes, only %d ready",
						SBData.Tag, SBData.DataSize,result);
			SB_PutString(buf);
			HandleCommError();
			return;
		}
		
		/* if we get here then all the message data is in the 
			SBData struct, and ready to be acted upon */
		HandleSBMessage();
	}

}		

	
/*---------------------------------------------------------------------------
 ** HandleSBMessage()
 
 *  PARAMETERS:		None (uses the global SBData struct)		

 *  DESCRIPTION:
 		Routes each message from the board to the appropriate routine
 		that handles that message. In most cases, the message handling
 		routine will be in sbmsg.c.	
 
 *  RETURNS:	Nothing
 		
--------------------------------------------------------------------------*/
void HandleSBMessage()
{
	
	switch (SBData.Tag)
	{                  
	
		case TAG_PENDOWN:
			OnPenDown();
			SB_DeltaDrawMode = TRUE;
			break;

		case TAG_DRAW:
			OnDraw();
			SB_DeltaDrawMode = TRUE;
			break;

		case TAG_PENUP:
			OnPenUp();
			SB_DeltaDrawMode = FALSE;
			break;

		case TAG_DELTA_DRAW:
			OnDeltaDraw();
			SB_DeltaDrawMode = TRUE;
			break;

		case TAG_SOFTBOARD_EXTENTS:
			OnSoftBoardExtents();
			break;

		case TAG_RECORDING_STARTED:
			SBStatus.Recording = TRUE;
			OnRecordingStarted();
			break;

		case TAG_RECORDING_STOPPED:
			SBStatus.Recording = FALSE;
			OnRecordingStopped();
			break;

		case TAG_SOFTBOARD_STATUS:
			/* download flag is used for board status check on app startup */
			SBStatus.DownloadDone = TRUE;
			/* check byte 8 for status */
			/* if bit 0 is set, we have are recording */
			if( SBData.Data[8] & 0x01 )
				SBStatus.Recording = TRUE;	
			else
				SBStatus.Recording = FALSE;
			 
			OnSoftBoardStatus();
			break;

		case TAG_MANUFACTURING_INFO:		
			OnManufacturingInfo();
			break;

		case TAG_DSPCODE_VERSION:		
			OnDSPCodeVersion();
			break;

		case TAG_SOFTBOARD_ALIGNMENT:		
			OnSoftBoardAlignment();
			break;

		case TAG_STANDBY_MODE_ENTERED:	
			OnStandbyModeEntered();
			break;

		case TAG_IDEAL_HEIGHTS:			
			OnIdealHeights();
			break;

		case TAG_SOFTBOARD_FLATNESS:
			OnSoftBoardFlatness();
			break;

		case TAG_ACTUAL_HEIGHTS:			
			OnActuaHeights();
			break;

		case TAG_END_OF_STARTUP_SEQUENCE:
			OnEndOfStartupSequence();
			break;

		case TAG_ECHO_STANDBY_IDLE_TIME:	
			OnEchoStandbyIdleTime();
			break;
			
		case TAG_TRACKING_RESUMED:
			OnTrackingResumed();
			break;

		default :
			/* tell someone we had a message we didn't recognize */
			ShowBadTag();
			break;
	}

}	


/*---------------------------------------------------------------------------
 ** ShowBadTag()
 
 *  PARAMETERS:		None (uses the global SBData struct)

 *  DESCRIPTION:	
 		Handles the case of an unknown in a message recieved from the 
 		SoftBoard. In most cases, just a debug message is shown.
 
 *  RETURNS:       Nothing.
 		
--------------------------------------------------------------------------*/
void ShowBadTag()
{
	char buf[SB_MAX_STR_LEN];

	sprintf(buf,"Unknown Tag: %d", SBData.Tag);
	SB_ShowError(buf);
}



/*---------------------------------------------------------------------------
 ** SBDataIsDownloadDone()
 
 *  PARAMETERS:	 None

 *  DESCRIPTION:	    
 		Checks to see if the SoftBoard has active DSP code. In other
 		words, checks to see if a download will be necessary
 		 
 *  RETURNS:    TRUE, is the board is active.
 				FALSE, if the board is not active, and needs a download
 				of the DSP code. 		     
 				
--------------------------------------------------------------------------*/
BOOL SBDataIsDownloadDone()
{
	char ErrorString[SB_MAX_STR_LEN];
	int result;
	
	/*  flush the buffers */
	SB_PutString("Flushing Buffers");
	SBCommFlush();
	if ( SBCommIsError() )
	{
		SBCommGetErrorString(ErrorString);
		SB_ShowError(ErrorString);
		SBCommClearError();
		return(FALSE);
	}                   
	
	SBStatus.DownloadDone = FALSE;

	//	SB_PutString("Sending TAG_QUERY_SOFTBOARD_STATUS");
	SBDataSendCommand(TAG_QUERY_SOFTBOARD_STATUS, 0);

  	/* check the port to see if SoftBoard sent any data back */
	result = SBCommByteReadyWait(1000);
	//	fprintf(stderr, "Reply: %d\n", result);
	if( (result == 0) || (result == COMM_TIMEOUT) )
	{
		/* SoftBoard is not responding */
		return(FALSE);
	}
	else if( result < 0)
	{
		SBCommGetErrorString(ErrorString);
		SB_ShowError(ErrorString);
		SBCommClearError();
		return(FALSE);
	}
	else /* we have some data waiting */
	{
		SBData.Busy = TRUE;
		GetSBData(); /* get a message from the SoftBoard and handle it */
		SBData.Busy = FALSE;

		/* What we expect to happen is to get a Softboard Status message, 
		   which will set the DownloadDone flag to true. Otherwise the 
		   board is dead, and the data coming back was garbage. In 
		   either case, DownloadDone will be accurate now. */
		return( SBStatus.DownloadDone );
	}
}


/*-------------------------------------------------------------------------*/
/* A debugging routine that prints out the command sent, and any data */
void ShowCommandSent( short Tag, BYTE *Data)
{
	char buf[80], tagstr[40];   
 
	switch( Tag )
	{
		case  TAG_START_RECORDING:
				strcpy(tagstr, "TAG_START_RECORDING");
				break;

		case  TAG_STOP_RECORDING:		   
				strcpy(tagstr, "TAG_STOP_RECORDING");
				break;

		case  TAG_START_ALIGNMENT_CHECK:
				strcpy(tagstr, "TAG_START_ALIGNMENT_CHECK");
				break;

		case  TAG_STOP_ALIGNMENT_CHECK:   
				strcpy(tagstr, "TAG_STOP_ALIGNMENT_CHECK");
				break;

		case  TAG_SET_STANDBY_IDLE_TIME:	   
				sprintf(tagstr, "TAG_SET_STANDBY_IDLE_TIME, Data: %u, %u",
							 Data[0], Data[1]);
				break;

		case  TAG_QUERY_SOFTBOARD_EXTENTS:
				strcpy(tagstr, "TAG_QUERY_SOFTBOARD_EXTENTS");
				break;

		case  TAG_QUERY_SOFTBOARD_STATUS:
				strcpy(tagstr, "TAG_QUERY_SOFTBOARD_STATUS");
				break;

		case  TAG_QUERY_MANUFACTURING_INFO:
				strcpy(tagstr, "TAG_QUERY_MANUFACTURING_INFO");
				break;

		case  TAG_QUERY_DSPCODE_VERSION:
				strcpy(tagstr, "TAG_QUERY_DSPCODE_VERSION");
				break;

		case  TAG_QUERY_SOFTBOARD_ALIGNMENT:
				strcpy(tagstr, "TAG_QUERY_SOFTBOARD_ALIGNMENT");
				break;

		case  TAG_QUERY_IDEAL_HEIGHTS:
				strcpy(tagstr, "TAG_QUERY_IDEAL_HEIGHTS");
				break;

		case  TAG_GOTO_STANDBY_MODE:
				strcpy(tagstr, "TAG_GOTO_STANDBY_MODE");
				break;
	
		case  TAG_SEND_STARTUP_SEQUENCE:
				strcpy(tagstr, "TAG_SEND_STARTUP_SEQUENCE");
				break;

		default:
				strcpy(tagstr, "UNKNOWN COMMAND TAG");
				break;

	}

	sprintf(buf, "Sending Command: %s", tagstr);
	SB_PutString(buf);
	
}



