/**************************************************************************
 *
 *    FILE:			sbmsg.c
 *
 *    DESCRIPTION:	Routines to handle the messages from the SoftBoard.
 *			 		The tag value and data are stored in the global
 *					SBData data structure by routines in sbdata.c,
 *					and the appropriate routine here is called,
 *					depending on the tag value. 
 *					
 *					The routine names are essentially OnTagName(), 
 *					using something very close to the tag name, 
 *					as defined in sbtags.h.
 *					
 *					These routines parse the data structure to extract the 
 *					relevant information for each message. Currently,
 *					the information is merely printed out, but you
 *					can easily add your own application-specific
 *					code to do whatever you need to do in response
 *					to messages from the SoftBoard.
 *
 *    HISTORY:    	
 *		Sept. 8, 1994:	First Release - Guy Townsend / Microfield
 *
 *		Sept. 22, 1994:	Added sbhw.h include, needed for NEXT, and added
 *						to some comments.
 *
 *		Oct. 6, 1994:	Fixed error in TAG_SOFTBOARD_ALIGNMENT handler.
 *						Message for level 4 error was incorrect.
 *
 *		May 12, 1995:	Added OnFatalCommError() handler, to notify application
 *						that a new download will be required to talk to 
 *						the SoftBoard again.
 *
 *		July 28, 1995:	Added ErrorString parameter to OnFatalCommError()
 *
 *		Oct. 11, 1995:	Added OnTrackingResumed() function.
 *		
 **************************************************************************/

#include <string.h>
#include <stdio.h>

#include "sbhw.h"
#include "sbmsg.h"
#include "sbdata.h"
#include "sbout.h"
#include "sbtags.h"
#include "sbdown.h"


/* TAG_PENDOWN handler (essentially the same as a move operation ) */
void OnPenDown()
{
	char buf[80];
	char pen[20];

	unsigned short low, high, x, y; 
	unsigned long loword, hiword, time;
	
	low = SBData.Data[0];
	high = SBData.Data[1];
	loword = (high << 8) | low;
	
	low = SBData.Data[2];
	high = SBData.Data[3];
	hiword = (high << 8) | low;

	time = (hiword << 16) | loword;  /* time is in milliseconds */       

	low = SBData.Data[4];
	high = SBData.Data[5];
	x = (high << 8) | low;
	
	low = SBData.Data[6];
	high = SBData.Data[7];
	y = (high << 8) | low;
	
	switch (SBData.Data[8])
	{
	case SBPEN_BLACK:			
	  strcpy(pen, "black");
	  break;
	  
	case SBPEN_RED:		
	  strcpy(pen, "red");
	  break;
	  
	case SBPEN_GREEN:
	  strcpy(pen, "green");
	  break;

	case SBPEN_BLUE:
	  strcpy(pen, "blue");
	  break;

	case SBPEN_LARGE_ERASER:
	  strcpy(pen, "leraser");
	  break;

	case SBPEN_SMALL_ERASER:
	  strcpy(pen, "seraser");
	  break;
	}	
	strcpy(outmesg, "");
	sprintf(outmesg, "PENDOWN:%s:%lu:%03d:%03d:",pen, time, x, y);
	send(outsocket, outmesg, sizeof(outmesg), 0);
	
	sprintf(buf, "Received TAG_PENDOWN, Time: %lu ms, X: %03d, Y: %03d, %s", time, x, y, pen);
    SB_PutString(buf);
}


/* TAG_DRAW handler */
void OnDraw()
{
	char buf[80];
	unsigned short low, high, x, y;
	
	low = SBData.Data[0];
	high = SBData.Data[1];
	x = (high << 8) | low;
	
	low = SBData.Data[2];
	high = SBData.Data[3];
	y = (high << 8) | low;

	strcpy(outmesg, "");
	sprintf(outmesg, "DRAW:%03d:%03d:", x, y);
	send(outsocket, outmesg, sizeof(outmesg), 0);

	sprintf(buf, "Received TAG_DRAW X: %03d, Y: %03d", x, y);
	SB_PutString(buf);
	
}


/* TAG_PENUP handler */
void OnPenUp()
{
  strcpy(outmesg, "");
  sprintf(outmesg, "PENUP");
  send(outsocket, outmesg, sizeof(outmesg), 0);
  
  SB_PutString("Received TAG_PENUP");
}


/* A delta draw command was received. Delta draw is not a separate tag 
   value that is received from the SoftBoard, but we treat it that way 
   for simplicity. See the code in sbdata.c for more details. */
/* You may want to handle delta draws differently from delta draws, when
	adding to your display list, so having two routines may help.*/
void OnDeltaDraw()
{
	char buf[80];
	signed char x, y;
	
	x = SBData.Data[0];
	y = SBData.Data[1];

	strcpy(outmesg, "");
	sprintf(outmesg,"DELTADRAW:%03d:%03d:", x, y);
	send(outsocket, outmesg, sizeof(outmesg), 0);

	sprintf(buf, "Received Delta Draw msg, X: %03d, Y: %03d", x, y);
    SB_PutString(buf);
	
}

/* There is no Leave Delta Draw message, but it was needed for the logic
   flow in sbdata.c, and is here in case there is a need for it at the 
   application level, such as in the construction of a display list. */
void OnLeaveDeltaDrawMode()
{
  strcpy(outmesg, "");
  sprintf(outmesg, "LEAVEDELTADRAW");
  send(outsocket, outmesg, sizeof(outmesg), 0);
  
  SB_PutString("Received Leaving Delta Draw msg");
}

 
/* TAG_SOFTBOARD_EXTENTS handler */
/* Sets the logical coordinate system of the SoftBoard */
void OnSoftBoardExtents()
{
	char buf[80];
	unsigned short low, high, left, right, top, bottom, width;
	
	low = SBData.Data[0];
	high = SBData.Data[1];
	left = (high << 8) | low;
	
	low = SBData.Data[2];
	high = SBData.Data[3];
	top = (high << 8) | low;

	low = SBData.Data[4];
	high = SBData.Data[5];
	right = (high << 8) | low;

	low = SBData.Data[6];
	high = SBData.Data[7];
	bottom = (high << 8) | low;

	low = SBData.Data[8];
	high = SBData.Data[9];
	width = (high << 8) | low;

	/* At this point, the logical coordinates will usually be stored
	   somewhere so they can be used in scaling the draw commands
	   to the user screen. */

	strcpy(outmesg, "");
	sprintf(outmesg, "SOFTBOARD_EXTENTS:%u:%u:%u:%u:%u", left, right, top, bottom, width);
	send(outsocket, outmesg, strlen(outmesg), 0);
    SB_PutString("Received TAG_SOFTBOARD_EXTENTS");
	sprintf(buf, "  Left: %u, Right: %u, Top: %u, Bottom: %u, Width: %u",
			left, right, top, bottom, width);
    SB_PutString(buf);
}


/* TAG_RECORDING_STARTED handler */
void OnRecordingStarted()
{
	SB_PutString("Received TAG_RECORDING_STARTED");
}


/* TAG_RECORDING_STOPPED handler */
void OnRecordingStopped()
{
    SB_PutString("Received TAG_RECORDING_STOPPED");
}

/* TAG_TRACKING_RESUMED handler */
void OnTrackingResumed()
{
    SB_PutString("Received TAG_TRACKING_RESUMED");
}


/* TAG_SOFTBOARD_STATUS handler */
void OnSoftBoardStatus()
{
	/* check byte 8 for status */
	/* if bit 0 is set, we have are recording */
	if( SBData.Data[8] & 0x01 )
	{
		SB_PutString("Received TAG_SOFTBOARD_STATUS: Recording");		
	}	
	else
	{
		SB_PutString("Received TAG_SOFTBOARD_STATUS: Not Recording");
	}	
}


/* TAG_MANUFACTURING_INFO handler */
void OnManufacturingInfo()
{
	char buf[80];
	unsigned short low, high, model, rev;
	unsigned long loword, hiword, serial;
	
	low = SBData.Data[0];
	high = SBData.Data[1];
	model = (high << 8) | low; 

	low = SBData.Data[2];
	high = SBData.Data[3];
	rev = (high << 8) | low;

	low = SBData.Data[4];
	high = SBData.Data[5];
	loword = (high << 8) | low;
	
	low = SBData.Data[6];
	high = SBData.Data[7];
	hiword = (high << 8) | low;

	serial = (hiword << 16) | loword;        
    
    sprintf(buf, 
			"Received TAG_MANUFACTURING_INFO: Serial: %05lu, Model: %u, Rev: %u", 
    		serial, model, rev );
    SB_PutString(buf);
}


/* TAG_DSPCODE_VERSION handler */
void OnDSPCodeVersion()
{
	char buf[80];
	unsigned short major, minor;
	
	minor = SBData.Data[0];
	major = SBData.Data[1];

	sprintf(buf, "Received TAG_DSPCODE_VERSION: %02u.%02u,", 
				major, minor );
    SB_PutString(buf);
}


/* TAG_SOFTBOARD_ALIGNMENT handler */
/* User should get some kind of warning if the level is not zero. */
void OnSoftBoardAlignment()
{
	char buf[80], status_str[60];
	unsigned short status_level;
	
	strcpy(buf, "Received TAG_SOFTBOARD_ALIGNMENT: ");

	status_level = SBData.Data[0];
	switch(status_level)
	{
		case 0:		strcpy(status_str, "OKAY");
					break;
					
		case 1:		strcpy(status_str, "Slight Board Alignment Error");
					break;
					
		case 2:		strcpy(status_str, "Significant Board Alignment Error");
					break;
					
		case 3:		strcpy(status_str, "Slight Scanner Alignment Error");
					break;
					
		case 4:		strcpy(status_str, "Significant Scanner Alignment Error");
					break;
					
		case 5:		strcpy(status_str, "Severe Alignment Error; Inoperable");
					break;
					
		case 6:		strcpy(status_str, "Outdated DSP Code; Inoperable");
					break;

		default:	strcpy(status_str, "INVALID STATUS ARG");
	}
					
	strcat(buf, status_str);
    SB_PutString(buf);
}


/* TAG_STANDBY_MODE_ENTERED handler */
/*	Received in response to TAG_GOTO_STANDBY_MODE command, or if the 
	standby idle time has expired _while_ recording. Not received
	when the standy idle time expires but not recording. */
void OnStandbyModeEntered()
{
    SB_PutString("Received TAG_STANDBY_MODE_ENTERED");
}


/* TAG_IDEAL_HEIGHTS handler */
/* Ideal heights are the heights of the laser beams at the 
   calibration strips as they were set up at the factory. 
*/
void OnIdealHeights()
{
	char buf[80];
	unsigned short low, high;
	unsigned short upper_left, lower_left, upper_right, lower_right;
	
	low = SBData.Data[0];
	high = SBData.Data[1];
	upper_left = (high << 8) | low;
	
	low = SBData.Data[2];
	high = SBData.Data[3];
	lower_left = (high << 8) | low;

	low = SBData.Data[4];
	high = SBData.Data[5];
	upper_right = (high << 8) | low;

	low = SBData.Data[6];
	high = SBData.Data[7];
	lower_right = (high << 8) | low;

	/* At this point, the ideal heights will usually be stored
	   somewhere so they can be used during the adjustment process. */
    SB_PutString("Received TAG_IDEAL_HEIGHTS");
	sprintf(buf, "  UpperLeft: %u, UpperRight: %u, LowerLeft: %u, LowerRight: %u,",
			upper_left, upper_right, lower_left, lower_right);
    SB_PutString(buf);
}


/* TAG_SOFTBOARD_FLATNESS handler */
void OnSoftBoardFlatness()
{
	char buf[80];
	short flatness;
	
    flatness = SBData.Data[0];
    
    sprintf(buf, "Received TAG_SOFTBOARD_FLATNESS: %d", flatness);
    SB_PutString(buf);
}


/* TAG_ACTUAL_HEIGHTS handler */
/* Ideal heights are the heights of the laser beams at the 
   calibration strips as they were set up at the factory. 
   Ideally, they should be set up the same in the field, 
   using the TAG_QUERY_IDEAL_HEIGHTS command. */
void OnActuaHeights()
{
	char buf[80];
	unsigned short low, high;
	unsigned short upper_left, lower_left, upper_right, lower_right;
	
	low = SBData.Data[0];
	high = SBData.Data[1];
	upper_left = (high << 8) | low;
	
	low = SBData.Data[2];
	high = SBData.Data[3];
	lower_left = (high << 8) | low;

	low = SBData.Data[4];
	high = SBData.Data[5];
	upper_right = (high << 8) | low;

	low = SBData.Data[6];
	high = SBData.Data[7];
	lower_right = (high << 8) | low;

	/* The actual heights can now be used to compare with the ideal
	 	heights, remembering that a value of 0x8000 (-32,768)
	 	indicates that the value cannot be read by the DSP. If a
	 	value of 0x8000 is received, no value should be displayed. */
    SB_PutString("Received TAG_ACTUAL_HEIGHTS");
	sprintf(buf, "  UpperLeft: %u, UpperRight: %u, LowerLeft: %u, LowerRight: %u,",
			upper_left, upper_right, lower_left, lower_right);
    SB_PutString(buf);
}


/* TAG_END_OF_STARTUP_SEQUENCE handler */
void OnEndOfStartupSequence()
{

    SB_PutString("Received TAG_END_OF_STARTUP_SEQUENCE");
}


/* TAG_ECHO_STANDBY_IDLE_TIME handler */
void OnEchoStandbyIdleTime()
{
	char buf[80];
	unsigned short low, high, idle_time;
	
	low = SBData.Data[0];
	high = SBData.Data[1];
	idle_time = (high << 8) | low;

	sprintf(buf, "Received TAG_ECHO_STANDBY_IDLE_TIME: %02u", idle_time );
    SB_PutString(buf);
}


void OnFatalCommError( char *ErrorString )
{
	/* call error handling routine here, and show ErrorString */
	SB_ShowError("Fatal Comm Error: Can't Talk to SoftBoard");
}

