/*******************************************************************
 *
 *    FILE:			test.c
 *
 *    DESCRIPTION:	
 *		Platform independent code for a general program to test the
 *		ability to download to and interact with a SoftBoard. The user 
 *		interface code will be in a file that is platform dependent. 
 *		The main program calls the routines in this file to download 
 *		the DSP code to the SoftBoard, check if the download was 
 *		already done, and close the link between the host and the SoftBoard.
 *
 *		Note that the main program must also call the SBDataCheckPort()
 *		function in the sbdata.c file on a regular basis to read the 
 *		serial port and respond to the messages from the SoftBoard.
 *
 *    HISTORY:    	
 *		Sept. 7, 1994:	First released - Guy Townsend
 *
 *		Sept. 22, 1994:	Added some delays that allow things to run 
 * 						smoother during initial development. 
 *
 *******************************************************************/

/** include files **/ 
#include <string.h>

#include "sbhw.h"
#include "sbout.h"
#include "sbdata.h"
#include "sbcomm.h"
#include "sbdown.h"
#include "sbtags.h"
#include "test.h" 

/*	Download the DSP code to the SoftBoard, and leave it ready
	to talk to. The port is opened if it isn't already open.
	Returns TRUE if the download was successful. */
BOOL DoDownload( char * PortString )
{
	char ErrorString[SBCOMM_ERR_STR_MAX];
	int result;
	char buf[SBCOMM_ERR_STR_MAX];
	
	/* set these globals for debugging only */
	/* SBShowDebugOutput = TRUE; */
	/* SBDebugCommReads = TRUE; */
	
	/* make sure that we don't already have a different port open */
	if( SBStatus.PortOpen && (strcmp(SBStatus.Port, PortString) != 0) )
	{
		SB_ShowError("Can't use different serial port without restarting app");
		return(FALSE);
	}

	/* open the comm port, if it's not already open */
	if(!SBStatus.PortOpen)
	{
		result = SBCommOpen( PortString );
		if (result != COMM_NO_ERROR)
		{
			SB_PutString("Error Opening Comm Port");
			SBCommGetErrorString(buf);
			SB_ShowError(buf);
			SBCommClearError();
			return(FALSE);
		}
		else
		{
			SB_PutString("Opened Comm Port");
			SBStatus.PortOpen = TRUE;
			strcpy(SBStatus.Port, PortString);
		}
    }

	/*  flush the buffers */
	SBCommFlush();
	if ( SBCommIsError() )
	{
		SB_PutString("!! Flush of Buffers Failed");
		SBCommGetErrorString(buf);
		SB_ShowError(buf);
		SBCommClearError();
		SBCommClose();
		return(FALSE);
	}                   
	else
		SB_PutString("Flushed Buffers");

	
    /* Do the download operation. */
    /* Status of the operation will be reported by calling
       the SB_PutString function. */
	SB_PutString("Beginning Download");
	SBStatus.DownloadDone = FALSE;
    result = SBDownload( "boot1b.hex", "boot2b.hex", 
    			"softbd.dsp", ErrorString);
	if(result == FALSE)
	{
		SBStatus.DownloadDone = FALSE; /* needed by other routines */
	 	SBTrace("Download Failed");
	 	SB_PutString("Download Failed: ");
	 	SB_PutString(ErrorString); 
		SB_PutString("Closing Comm port");
		SBStatus.PortOpen = FALSE;
		SBCommClose();
	}
	else
	{
		SBStatus.DownloadDone = TRUE; /* needed by other routines */
		SB_PutString("Download Passed");
		/* wait for the board to spin up */
		/* We don't have to do this if we go directly into a loop
		   that will get all the data from the SoftBoard. It's only
		   done here because first step example programs may not
		   have the facility to sit and watch the port; but we
		   still want to get the initial startup data to verify
		   that the SoftBoard is actually running. */
		SBWait(5000); 
	}
		
	return(result);		
    
}    
    

/*	Check to see if the SoftBoard is already operational, or if a download 
	is required. Returns FALSE if communication with the board cannot be
	established. */
BOOL CheckDownloadStatus( char * PortString )
{
	char ErrorString[SBCOMM_ERR_STR_MAX];
	int result;
	
	/* make sure that we don't already have a different port open */
	if( SBStatus.PortOpen && (strcmp(SBStatus.Port, PortString) != 0) )
	{
		SB_ShowError("Can't use different serial port without restarting app");
		return(FALSE);
	}

	/* open the comm port, if it's not already open */
	if(!SBStatus.PortOpen)
	{
		result = SBCommOpen( PortString );
		if (result != COMM_NO_ERROR)
		{
			SBCommGetErrorString(ErrorString);
			SB_ShowError(ErrorString);
			SBCommClearError();
			TestClose();
			return(FALSE);
		}
		else
		{
			SB_PutString("Opened Comm Port");
			SBStatus.PortOpen = TRUE;
			strcpy(SBStatus.Port, PortString);
		}
    }
    
    /* set the baud rate to normal operation rate */
    SBCommSetBaudRate( 9600 );

	SB_PutString("Checking SoftBoard Status");

	/* call routine to check download status, and report results */
	if( SBDataIsDownloadDone() )
	{
		SB_PutString("SoftBoard is active");
		return(TRUE);
	}
	else
	{
		SB_PutString("SoftBoard is not active");
		TestClose();
		return(FALSE);
	}
    
}    
    

/*	Close the serial port, and any other cleanup needed 
	before closing the app.*/
void TestClose()
{
	BYTE time[2];
	
	time[0] = 1; /* one second timeout */
	time[1] = 0;
	
	
	/*  close the port, if it's open */
	if(SBStatus.PortOpen)
	{
		if(SBStatus.DownloadDone)
		{
			/* first shut down the scanner motors on the SoftBoard */
			SBDataSendCommand(TAG_GOTO_STANDBY_MODE, 0);               
			/* wait a bit to make sure the command gets to the board
				and we see the message back from the board before we 
				close the port */
			SBWait(1000);      /* wait for SB to respond */
			SBDataCheckPort(); /* get the board's response */
			SBWait(2000); /* wait to get the response on the screen */
			/* we reset this flag to keep any remaining timer messages from
			 	triggering accesses to the SoftBoard when the port is closed.   */
			SBStatus.DownloadDone = FALSE;
		}
		SBCommFlush(); /* so close() doesn't hang in case there is data in the buffers */
		if( SBCommClose() != 0)
			SB_ShowError("!! Error closing comm port");
		else
		{
	    	 SBTrace("Closed Comm Port");
	    	 SB_PutString("Closed Comm Port");
	    	 SBStatus.PortOpen = FALSE;
	    }
	    /* hang around so we can see the last messages on the screen */
	    SBWait(2000);  
    }
}


    

